###############################################################
# Name:             Preferences.py
# Purpose:     Hold program preferences
# Author:         Luca Allulli (webmaster@roma21.it)
# Created:     2010-01-31
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:     GNU GPL v2
##############################################################

import datetime

from . import i18n
from .SongFormat import *
from .decorators import StandardVerseNumbers
from .Transpose import *

_ = wx.GetTranslation


def get_update_frequencies():
    return {
        0: _("Never"),
        7: _("Week"),
        14: _("Two weeks"),
        30: _("Month"),
        60: _("Two months"),
    }
    
class Preferences(object):
    """
        Available preferences

        * format
        * decoratorFormat
        * decorator
        * chorusLabel (None or string)
        * labelVerses
        * editorFace
        * editorSize
        * fontFace
        * defaultNotation
        * notations
        * defaultNotation
        * autoAdjustSpuriousLines
        * autoAdjustTab2Chordpro
        * autoAdjustEasyKey
        * locale
        * updateFrequency (days, or 0 for never)
        * ignoredUpdates
        * updateUrl
        * updateLastCheck
        * Set/GetEasyChordsGroup
        * GetEasyChords
    """
    def __init__(self):
        object.__init__(self)
        self.config = wx.Config.Get()
        self.format = SongFormat()
        self.decoratorFormat = StandardVerseNumbers.Format(self.format, _("Chorus"))
        self.decorator = StandardVerseNumbers.Decorator(self.decoratorFormat)
        self.notations = [enNotation, itNotation, itUcNotation, deNotation, tradDeNotation, frNotation, ptNotation]
        self.easyChordsGroup = {}
        self.Load()

    def SetFont(self, font, showChords=None):
        self.fontFace = font
        self.format.face = font
        self.format.comment.face = font
        self.format.chord.face = font
        self.format.chorus.face = font
        self.format.chorus.chord.face = font
        self.format.chorus.comment.face = font
        for v in self.format.verse:
            v.face = font
            v.chord.face = font
            v.comment.face = font
        self.format.title.face = font
        self.format.subtitle.face = font
        self.decoratorFormat.face = font
        self.decoratorFormat.chorus.face = font
        if showChords is not None:
            self.format.showChords = showChords

    def Load(self):
        self.notices = {}
        self.config.SetPath('/Format')
        l = self.config.Read('ChorusLabel')
        if l:
            self.chorusLabel = l
            self.decoratorFormat.SetChorusLabel(l)
        else:
            self.chorusLabel = None
        showChords = int(self.config.Read('ShowChords', "2"))
        self.config.SetPath('/Format/Font')
        face = self.config.Read('Face')
        if face:
            self.fontFace = face
        else:
            self.fontFace = "Arial"
        self.SetFont(self.fontFace, showChords)
        self.config.SetPath('/Format/Style')
        labelVerses = self.config.Read('LabelVerses')
        if labelVerses:
            self.labelVerses = bool(int(labelVerses))
        else:
            self.labelVerses = True
        self.config.SetPath('/Editor')
        self.editorFace = self.config.Read('Face')
        self.editorSize = self.config.Read('Size')
        if not self.editorFace:
            self.editorFace = wx.SystemSettings().GetFont(wx.SYS_ANSI_FIXED_FONT).GetFaceName() # "Lucida Console"
            self.editorSize = 12
        else:
            self.editorSize = int(self.editorSize)
        n = self.config.Read('DefaultNotation')
        if n:
            self.defaultNotation = n
            self.notations = [x for x in self.notations if x.id == n] + [x for x in self.notations if x.id != n]
        else:
            lang = i18n.getLang()
            self.defaultNotation = None
            if lang in defaultLangNotation:
                n = defaultLangNotation[lang].id
                self.notations = [x for x in self.notations if x.id == n] + [x for x in self.notations if x.id != n]
        self.config.SetPath('/AutoAdjust')
        spuriousLines = self.config.Read('spuriousLines')
        if spuriousLines:
            self.autoAdjustSpuriousLines = bool(int(spuriousLines))
        else:
            self.autoAdjustSpuriousLines = True
        tab2chordpro = self.config.Read('tab2chordpro')
        if tab2chordpro:
            self.autoAdjustTab2Chordpro = bool(int(tab2chordpro))
        else:
            self.autoAdjustTab2Chordpro = True
        easyKey = self.config.Read('easyKey')
        if easyKey:
            self.autoAdjustEasyKey = bool(int(easyKey))
        else:
            self.autoAdjustEasyKey = False
            self.notices['firstTimeEasyKey'] = True
        self.config.SetPath('/AutoAdjust/EasyChordsGroups')
        for k in easyChordsOrder:
            l = self.config.Read(k)
            if l:
                l = int(l)
            else:
                l = easyChords[k][2]
            self.SetEasyChordsGroup(k, l)
        self.config.SetPath('/App')
        ext = self.config.Read('defaultExtension')
        if not ext:
            self.defaultExtension = 'crd'
        else:
            self.defaultExtension = ext
        lang = self.config.Read('locale')
        if not lang:
            self.locale = None
        else:
            self.locale = lang
        self.config.SetPath('/AutoUpdate')
        f = self.config.Read('frequency')
        if not f:
            self.updateFrequency = 7
        else:
            self.updateFrequency = int(f)
        i = self.config.Read('ignored')
        if not i:
            self.ignoredUpdates = set()
        else:
            self.ignoredUpdates = set(i.split(','))
        u = self.config.Read('url')
        if not u:
            self.updateUrl = 'https://songpress.skeed.it/xmlrpc'
        else:
            self.updateUrl = u
        d = self.config.Read('lastCheck')
        if not d:
            self.updateLastCheck = None
        else:
            self.updateLastCheck = datetime.datetime.fromordinal(int(d))

    def Bool2String(self, param):
        return "1" if param else "0"

    def Save(self):
        self.config.SetPath('/Format')
        if self.chorusLabel is not None:
            self.config.Write('ChorusLabel', self.chorusLabel)
        self.config.Write('ShowChords', str(self.format.showChords))
        self.config.SetPath('/Format/Font')
        face = self.config.Write('Face', self.fontFace)
        self.config.SetPath('/Format/Style')
        self.config.Write('LabelVerses', self.Bool2String(self.labelVerses))
        self.config.SetPath('/Editor')
        self.config.Write('Face', self.editorFace)
        self.config.Write('Size', str(self.editorSize))
        if self.defaultNotation is not None:
            self.config.Write('DefaultNotation', self.defaultNotation)
        self.config.SetPath('/AutoAdjust')
        self.config.Write('spuriousLines', self.Bool2String(self.autoAdjustSpuriousLines))
        self.config.Write('tab2chordpro', self.Bool2String(self.autoAdjustTab2Chordpro))
        self.config.Write('easyKey', self.Bool2String(self.autoAdjustEasyKey))
        self.config.SetPath('/AutoAdjust/EasyChordsGroups')
        for k in easyChordsOrder:
            self.config.Write(k, str(self.GetEasyChordsGroup(k)))
        self.config.SetPath('/App')
        self.config.Write('defaultExtension', self.defaultExtension)
        if self.locale is not None:
            lang = self.config.Write('locale', self.locale)
        self.config.SetPath('/AutoUpdate')
        self.config.Write('frequency', str(self.updateFrequency))
        self.config.Write('ignored', ",".join(self.ignoredUpdates))
        self.config.Write('url', self.updateUrl)
        if self.updateLastCheck is not None:
            self.config.Write('lastCheck', str(self.updateLastCheck.toordinal()))
        self.config.Flush()

    def SetChorusLabel(self, c):
        self.chorusLabel = c
        self.decoratorFormat.SetChorusLabel(c)

    def SetDefaultNotation(self, notation):
        self.defaultNotation = notation
        self.notations = [x for x in self.notations if x.id == notation] + [x for x in self.notations if x.id != notation]

    def SetEasyChordsGroup(self, group, level):
        self.easyChordsGroup[group] = level
        self.easyChords = None

    def GetEasyChordsGroup(self, group):
        return self.easyChordsGroup[group]

    def GetEasyChords(self):
        if self.easyChords is None:
            self.easyChords = {'Eb': -1}
            for k in easyChords:
                chords = easyChords[k][1]
                l = self.easyChordsGroup[k] / 4.0
                for c in chords:
                    if c in self.easyChords:
                        self.easyChords[c] = max(self.easyChords[c], l)
                    else:
                        self.easyChords[c] = l

        return self.easyChords



from contextlib import contextmanager
import tempfile
import logging
import shutil


@contextmanager
def temp_dir(keep=False):
    """
    Context manager. Create and yield a temporary dir, and destroys it on exit
    """
    t = tempfile.mkdtemp()
    try:
        yield t
    finally:
        if keep:
            logging.info(f"Keeping temporary folder: {t}")
        else:
            shutil.rmtree(t)



###############################################################
# Name:             PreviewCanvas.py
# Purpose:     Window containing preview
# Author:         Luca Allulli (webmaster@roma21.it)
# Created:     2009-02-21
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:     GNU GPL v2
##############################################################

import platform

import wx
import wx.adv

from .Renderer import *


_ = wx.GetTranslation


class PreviewCanvas(object):
    def __init__(self, parent, sf, notations, sd=SongDecorator(), embedded=False):
        object.__init__(self)
        self.main_panel = wx.Window(parent)
        bSizer = wx.BoxSizer(wx.VERTICAL)
        parent = self.main_panel
        if not embedded:
            self.link = wx.adv.HyperlinkCtrl(parent, 0, _("Copy formatted song to clipboard"), '')
            tt = wx.ToolTip(_("Copy formatted song to clipboard, so that it can be pasted in any program and printed"))
            self.link.SetToolTip(tt)
            bSizer.Add(self.link, 0, wx.EXPAND)
        self.panel = wx.ScrolledWindow(parent, style=wx.BORDER_DOUBLE)
        self.panel.SetBackgroundStyle(wx.BG_STYLE_CUSTOM)
        self.pixedScrolled = 10
        self.panel.SetScrollbars(self.pixedScrolled, self.pixedScrolled, 0, 0)
        self.panel.Bind(wx.EVT_PAINT, self.OnPaint, self.panel)
        self.panel.SetBackgroundColour(wx.WHITE)
        self.text = ""
        bSizer.Add(self.panel, 1, wx.EXPAND)
        #SongFormat
        self.renderer = Renderer(sf, sd, notations)
        self.main_panel.SetSizer(bSizer)
        self.main_panel.Layout()


    def OnPaint(self, e):
        #print("OnPaint")
        dc = wx.AutoBufferedPaintDC(self.panel)
        self.panel.PrepareDC(dc)
        dc.SetBackground(wx.WHITE_BRUSH)
        dc.Clear()
        w, h = self.renderer.Render(self.text, dc)
        self.panel.SetVirtualSize(wx.Size(int(w), int(h)))
        

    def Refresh(self, text):
        self.text = text
        self.panel.Refresh()

    def SetDecorator(self, sd):
        self.renderer.SetDecorator(sd)



###############################################################
# Name:             SongDecorator.py
# Purpose:     Base (and default) handlers for verse and chorus
# Author:         Luca Allulli (webmaster@roma21.it)
# Created:     2009-02-21
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:     GNU GPL v2
##############################################################

from .SongFormat import *
from .SongBoxes import *


class SongDecorator(object):
    def __init__(self):
        object.__init__(self)
        self.dc = None
        # Current y
        self.y = 0
        self.dc = None
        self.firstBlockOffsetY = 0
        self.lastBlockOffsetY = 0
        # SongBox
        self.s = None
        
    def SetMarginText(self, text):
        # Modify text margins
        pass
        
    def SetMarginChord(self, chord):
        # Modify chord margins
        pass
        
    def SetMarginLine(self, line):
        # Modify line margins
        pass

    def SetMarginBlock(self, block):
        # Modify line margins
        pass

    def SetMarginSong(self, song):
        # Modify line margins
        pass
        
    def LayoutComposeLine(self, line):
        # Pass 1: determine size of text
        chordMaxH = 0
        chordMaxTH = 0
        textMaxH = 0
        textMaxTH = 0
        chordsOnly = True
        hasChords = False
        for t in line.boxes:
            self.dc.SetFont(t.font)
            text = t.text
            t.w, t.h = self.dc.GetTextExtent(text)
            if t.type == SongText.chord:
                hasChords = True
                self.SetMarginChord(t)
                chordMaxH = max(chordMaxH, t.h)
                chordMaxTH = max(chordMaxTH, t.GetTotalHeight())
                t.w += self.dc.GetTextExtent(" ")[0] / 2
            else:
                if text.strip() != '':
                    chordsOnly = False
                self.SetMarginText(t)
                textMaxH = max(textMaxH, t.h)
                textMaxTH = max(textMaxTH, t.GetTotalHeight())
        if chordsOnly and hasChords:
            textMaxH = 0
            textMaxTH = 0
            line.textBaseline = chordMaxTH
        elif chordsOnly: # Block without text or chords
            w, h = self.dc.GetTextExtent(' ')
            textMaxH = h
            textMaxTH = h
            line.textBaseline = h
        else:
            line.textBaseline = chordMaxTH + chordMaxH * (line.parent.format.chordSpacing - 1) + textMaxTH
        line.chordBaseline = chordMaxTH
        
        line.h = line.textBaseline + textMaxH * (line.parent.format.textSpacing - 1)
        # Pass 2: set layout
        x = 0
        chordX = 0
        for t in line.boxes:
            self.dc.SetFont(t.font)
            if t.type == SongText.chord:
                t.x = max(x, chordX)
                x = t.x
                chordX = x + t.GetTotalWidth()
                t.y = line.chordBaseline - t.GetTotalHeight()
            else:
                t.x = x
                x_chord = max(x, chordX)
                x = t.x + t.GetTotalWidth()
                t.y = line.textBaseline - t.GetTotalHeight()
                if t.text.strip() == '':
                    chordX = x_chord + t.GetTotalWidth()
            line.RelocateBox(t)
        self.SetMarginLine(line)
        
    def LayoutComposeBlock(self, block):
        y = 0
        for l in block.boxes:
            l.y = y
            y += l.GetTotalHeight()
            block.RelocateBox(l)
        self.SetMarginBlock(block)

    def LayoutComposeSong(self, song):
        y = 0
        self.dc.SetFont(song.format.wxFont)
        for b in song.boxes:
            b.y = y
            y += b.GetTotalHeight() + b.GetLastLineTextHeight() * song.format.blockSpacing
            song.RelocateBox(b)
        self.SetMarginSong(song)

    def LayoutCompose(self):
        # Postorder layout composing
        for block in self.s.boxes:
            for line in block.boxes:
                self.LayoutComposeLine(line)
        for block in self.s.boxes:
            self.LayoutComposeBlock(block)                
        self.LayoutComposeSong(self.s)
        
    def LayoutMoveBlock(self, block):
        # Move block within song
        pass
        
    def LayoutMoveLine(self, line):
        # Move line within block
        # If we need to, we can even move text and chords inside this line
        pass

    def LayoutMove(self):
        # Now that sizes are set, we can move elements inside each box if we need to
        for block in self.s.boxes:
            # Move block within song
            self.LayoutMoveBlock(block)
            for line in block.boxes:
                # Move line within block
                # If we need to, we can even move text and chords inside this line
                self.LayoutMoveLine(line)
                
    def PreDrawSong(self, song):
        pass
        
    def PreDrawBlock(self, block, bx, by):
        # bx, by: coordinates of top-left corner of drawable area
        pass
        
    def PreDrawLine(self, line, lx, ly):
        # lx, ly: coordinates of top-left corner of drawable area
        pass
        
    def PreDrawText(self, text, tx, ty):
        # tx, ty: coordinates of top-left corner of drawable area
        pass
        
    def DrawText(self, text, tx, ty):
        # tx, ty: coordinates of top-left corner of drawable area
        self.dc.SetFont(text.font)
        self.dc.SetTextForeground(text.color)
        self.dc.DrawText(text.text, int(tx + text.marginLeft), int(ty + text.marginTop))
        
    def PostDrawText(self, text, tx, ty):
        # tx, ty: coordinates of top-left corner of drawable area
        pass        
        
    def PostDrawLine(self, line, lx, ly):
        # lx, ly: coordinates of top-left corner of drawable area
        pass        
        
    def PostDrawBlock(self, block, bx, by):
        # bx, by: coordinates of top-left corner of drawable area
        pass
        
    def PostDrawSong(self, song):
        pass
        
    def DrawBoxes(self):
        if self.s.drawWholeSong:
            self.PreDrawSong(self.s)
            firstBlock = False
        else:
            firstBlock = True
        for block in self.s.boxes:
            block: SongBlock
            if block.drawBlock:
                if firstBlock:
                    firstBlock = False
                    self.firstBlockOffsetY = self.s.marginTop + block.y
                bx = self.s.marginLeft + block.x
                by = self.s.marginTop + block.y - self.firstBlockOffsetY
                self.PreDrawBlock(block, bx, by)
                for line in block.boxes:
                    lx = bx + block.marginLeft + line.x
                    ly = by + block.marginTop + line.y
                    self.PreDrawLine(line, lx, ly)
                    for text in line.boxes:
                        tx = lx + line.marginLeft + text.x
                        ty = ly + line.marginTop + text.y
                        self.PreDrawText(text, tx, ty)
                        self.DrawText(text, tx, ty)
                        self.PostDrawText(text, tx, ty)
                self.lastBlockOffsetY = by + block.GetTotalHeight()
                self.PostDrawBlock(block, bx, by)
        if self.s.drawWholeSong:
            self.PostDrawSong(self.s)
        
    def InitDraw(self):
        pass
        
    def Draw(self, s: SongSong, dc):
        self.s = s
        self.dc = dc
        self.InitDraw()
        self.LayoutCompose()
        self.LayoutMove()
        self.DrawBoxes()
        self.dc = None
        if self.s.drawWholeSong:
            w = self.s.GetTotalHeight()
        else:
            w = self.lastBlockOffsetY
        return self.s.GetTotalWidth(), w




# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version Mar 10 2018)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
_ = wx.GetTranslation

###########################################################################
## Class NotationDialog
###########################################################################

class NotationDialog(wx.Dialog):

    def __init__(self, parent):
        wx.Dialog.__init__(self, parent, id=wx.ID_ANY, title=_(u"Change notation"), pos=wx.DefaultPosition,
                                             size=wx.Size(361, 175), style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)

        self.SetSizeHints(wx.DefaultSize, wx.DefaultSize)

        bSizer1 = wx.BoxSizer(wx.VERTICAL)

        bSizer2 = wx.BoxSizer(wx.HORIZONTAL)

        self.label1 = wx.StaticText(self, wx.ID_ANY, _(u"From notation:"), wx.DefaultPosition, wx.DefaultSize, 0)
        self.label1.Wrap(-1)
        bSizer2.Add(self.label1, 0, wx.ALL, 5)

        fromNotationChoices = []
        self.fromNotation = wx.Choice(self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, fromNotationChoices, 0)
        self.fromNotation.SetSelection(0)
        bSizer2.Add(self.fromNotation, 1, wx.ALL, 5)

        bSizer1.Add(bSizer2, 1, wx.EXPAND, 5)

        bSizer21 = wx.BoxSizer(wx.HORIZONTAL)

        self.label11 = wx.StaticText(self, wx.ID_ANY, _(u"To notation:"), wx.DefaultPosition, wx.DefaultSize, 0)
        self.label11.Wrap(-1)
        bSizer21.Add(self.label11, 0, wx.ALL, 5)

        toNotationChoices = []
        self.toNotation = wx.Choice(self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, toNotationChoices, 0)
        self.toNotation.SetSelection(0)
        bSizer21.Add(self.toNotation, 1, wx.ALL, 5)

        bSizer1.Add(bSizer21, 1, wx.EXPAND, 5)

        m_sdbSizer1 = wx.StdDialogButtonSizer()
        self.m_sdbSizer1OK = wx.Button(self, wx.ID_OK)
        m_sdbSizer1.AddButton(self.m_sdbSizer1OK)
        self.m_sdbSizer1Cancel = wx.Button(self, wx.ID_CANCEL)
        m_sdbSizer1.AddButton(self.m_sdbSizer1Cancel)
        m_sdbSizer1.Realize();

        bSizer1.Add(m_sdbSizer1, 0, wx.ALL | wx.EXPAND, 5)

        self.SetSizer(bSizer1)
        self.Layout()

        # Connect Events
        self.fromNotation.Bind(wx.EVT_CHOICE, self.OnFromNotation)

    def __del__(self):
        pass

    # Virtual event handlers, overide them in your derived class
    def OnFromNotation(self, event):
        event.Skip()





###############################################################
# Name:             FontFaceDialog.py
# Purpose:     wxWidgets dialog with a FontComboBox
#            selector
# Author:         Luca Allulli (webmaster@roma21.it)
# Created:     2009-08-14
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:     GNU GPL v2
##############################################################

from .PreviewCanvas import *
from .FontComboBox import FontComboBox

_ = wx.GetTranslation


class FontFaceDialog(wx.Dialog):
    def __init__(self, parent, id, title, songFormat, songDecorator, decoratorFormat):
        self.format = songFormat
        self.originalFont = songFormat.face
        self.decoratorFormat = decoratorFormat
        wx.Dialog.__init__(self, parent, id, title, size=wx.Size(540, 320))

        bSizer1 = wx.BoxSizer(wx.VERTICAL)
        hSizer1 = wx.BoxSizer(wx.HORIZONTAL)
        
        m_staticText1 = wx.StaticText(self, wx.ID_ANY, _("Font"))
        m_staticText1.Wrap(-1);
        hSizer1.Add(m_staticText1, 0, wx.ALL, 5)
        
        self.fontCB = FontComboBox(self, wx.ID_ANY, songFormat.face)
        self.Bind(wx.EVT_COMBOBOX, self.OnFontSelected, self.fontCB)
        self.fontCB.Bind(wx.EVT_TEXT_ENTER, self.OnFontSelected, self.fontCB)
        self.fontCB.Bind(wx.EVT_KILL_FOCUS, self.OnFontSelected, self.fontCB)
        hSizer1.Add(self.fontCB, 1, wx.EXPAND | wx.ALL, 5)

        hSizer2 = wx.BoxSizer(wx.HORIZONTAL)
        
        m_staticText2 = wx.StaticText(self, wx.ID_ANY, _("Preview"))
        m_staticText2.Wrap(-1)
        hSizer2.Add(m_staticText2, 0, wx.ALL, 5)
        
        self.previewCanvas = PreviewCanvas(self, songFormat, songDecorator, embedded=True)
        hSizer2.Add(self.previewCanvas.main_panel, 1, wx.EXPAND | wx.ALL, 5)
        previewSong = _("{t:My Bonnie}\n\nMy [D]Bonnie lies [G]over the [D]ocean\noh [G]bring back my [A]Bonnie to [D]me!\n\n{soc}\n[D]Bring back, [E-]bring back,\n[A]bring back my Bonnie to [D]me!\n{eoc}")
        self.previewCanvas.Refresh(previewSong)
        
        bSizer1.Add(hSizer1, 0, wx.EXPAND, 5)
        bSizer1.Add(hSizer2, 1, wx.EXPAND, 5)
        
        bSizer2 = wx.BoxSizer(wx.HORIZONTAL)
        
        bSizer2.AddStretchSpacer(1)
        
        self.ok = wx.Button(self, wx.ID_OK, _("OK"))
        bSizer2.Add(self.ok, 0, wx.ALL, 5 )
        
        self.cancel = wx.Button(self, wx.ID_CANCEL, _("Cancel"))
        bSizer2.Add(self.cancel, 0, wx.ALL, 5)
        self.Bind(wx.EVT_BUTTON, self.OnCancel, self.cancel)
        
        bSizer1.Add(bSizer2, 0, wx.EXPAND, 5)
        
        self.SetSizer(bSizer1)
        self.Layout()
        
    def SetFont(self, font):
        self.format.face = font
        self.format.chord.face = font
        self.format.comment.face = font
        self.format.chorus.face = font
        self.format.chorus.chord.face = font
        self.format.chorus.comment.face = font
        for v in self.format.verse:
            v.face = font
            v.chord.face = font
            v.comment.face = font
        self.format.title.face = font
        self.decoratorFormat.face = font
        self.decoratorFormat.chorus.face = font
        previewSong = _("{t:My Bonnie}\n\nMy [D]Bonnie lies [G]over the [D]ocean\noh [G]bring back my [A]Bonnie to [D]me!\n\n{soc}\n[D]Bring back, [E-]bring back,\n[A]bring back my Bonnie to [D]me!\n{eoc}")        
        self.previewCanvas.Refresh(previewSong)    
        
    def OnFontSelected(self, evt):
        font = self.fontCB.GetValue()
        self.SetFont(font)
        evt.Skip()
        
    def GetValue(self):
        return self.fontCB.GetValue()
        
    def OnCancel(self, evt):
        font = self.originalFont
        self.SetFont(font)
        evt.Skip()
        



###############################################################
# Name:             main.py
# Purpose:     Entry point for Songpress
# Author:         Luca Allulli (webmaster@roma21.it)
# Created:     2009-01-16
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:     GNU GPL v2
##############################################################

import sys, os

import wx

from .Globals import glb
from . import i18n
from . import dev_tool


class SongpressApp(wx.App):
    def OnInit(self):
        self.SetAppName("songpress")
        glb.InitDataPath()
        self.config = wx.FileConfig(localFilename=os.path.join(glb.data_path, "config.ini"))
        wx.Config.Set(self.config)
        i18n.init(glb.default_language, [l for l in glb.languages])
        self.config.SetPath("/App")
        l = self.config.Read("locale")
        if l:
            print("Setting language ", l)
            i18n.setLang(l)
        else:
            print("Setting system language ")
            i18n.setSystemLang()

        from . import SongpressFrame
        self.res = wx.xrc.XmlResource(glb.AddPath("xrc/songpress.xrc"))
        songpressFrame = SongpressFrame.SongpressFrame(self.res)
        return True


def main():
    if "--create-shortcuts" in sys.argv:
        from .shortcuts import create_shortcuts
        create_shortcuts()
        return
    sys.excepthook = dev_tool.ExceptionHook
    songpressApp = SongpressApp()
    songpressApp.MainLoop()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

import wx

from .FontComboBox import FontComboBox
from . import Editor

_ = wx.GetTranslation

# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version Mar 10 2018)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc

###########################################################################
## Class PreferencesDialog
###########################################################################

class PreferencesDialog(wx.Dialog):

    def __init__(self, parent):
        wx.Dialog.__init__(self, parent, id=wx.ID_ANY, title=_(u"Songpress options"), pos=wx.DefaultPosition,
                                             size=wx.Size(535, 500), style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)

        self.SetSizeHints(wx.DefaultSize, wx.DefaultSize)

        bSizer10 = wx.BoxSizer(wx.VERTICAL)

        self.notebook = wx.Notebook(self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, 0)
        self.general = wx.Panel(self.notebook, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL)
        bSizer11 = wx.BoxSizer(wx.VERTICAL)

        bSizer12 = wx.BoxSizer(wx.HORIZONTAL)

        self.label1 = wx.StaticText(self.general, wx.ID_ANY, _(u"Editor font"), wx.DefaultPosition, wx.DefaultSize, 0)
        self.label1.Wrap(-1)
        bSizer12.Add(self.label1, 0, wx.ALL, 5)

        self.fontCB = FontComboBox(self.general, wx.ID_ANY, self.pref.editorFace)
        bSizer12.Add(self.fontCB, 1, wx.ALL, 5)

        self.m_staticText8 = wx.StaticText(self.general, wx.ID_ANY, _(u"Size"), wx.DefaultPosition, wx.DefaultSize, 0)
        self.m_staticText8.Wrap(-1)
        bSizer12.Add(self.m_staticText8, 0, wx.ALL, 5)

        sizeCBChoices = [_(u"7"), _(u"8"), _(u"9"), _(u"10"), _(u"11"), _(u"12"), _(u"13"), _(u"14"), _(u"16"), _(u"18"),
                                         _(u"20")]
        self.sizeCB = wx.ComboBox(self.general, wx.ID_ANY, _(u"12"), wx.DefaultPosition, wx.DefaultSize, sizeCBChoices, 0)
        self.sizeCB.SetMinSize(wx.Size(100, -1))

        bSizer12.Add(self.sizeCB, 0, wx.ALL, 5)

        bSizer11.Add(bSizer12, 0, wx.EXPAND, 5)

        bSizer13 = wx.BoxSizer(wx.HORIZONTAL)

        self.m_staticText9 = wx.StaticText(self.general, wx.ID_ANY, _(u"Preview"), wx.DefaultPosition, wx.DefaultSize, 0)
        self.m_staticText9.Wrap(-1)
        bSizer13.Add(self.m_staticText9, 0, wx.ALL, 5)

        self.editor = Editor.Editor(self.general, False, self.general)
        bSizer13.Add(self.editor, 1, wx.ALL | wx.EXPAND, 5)

        bSizer11.Add(bSizer13, 1, wx.EXPAND, 5)

        bSizer141 = wx.BoxSizer(wx.HORIZONTAL)

        self.m_staticText101 = wx.StaticText(self.general, wx.ID_ANY, _(u"Default notation"), wx.DefaultPosition,
                                                                                 wx.DefaultSize, 0)
        self.m_staticText101.Wrap(-1)
        bSizer141.Add(self.m_staticText101, 0, wx.ALL, 5)

        notationChChoices = []
        self.notationCh = wx.Choice(self.general, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, notationChChoices, 0)
        self.notationCh.SetSelection(0)
        bSizer141.Add(self.notationCh, 1, wx.ALL, 5)

        bSizer11.Add(bSizer141, 0, wx.EXPAND, 5)

        bSizer1411 = wx.BoxSizer(wx.HORIZONTAL)

        self.m_staticText1011 = wx.StaticText(self.general, wx.ID_ANY, _(u"Check for updates every"), wx.DefaultPosition,
                                                                                    wx.DefaultSize, 0)
        self.m_staticText1011.Wrap(-1)
        bSizer1411.Add(self.m_staticText1011, 0, wx.ALL, 5)

        frequencyChoices = []
        self.frequency = wx.Choice(self.general, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, frequencyChoices, 0)
        self.frequency.SetSelection(0)
        bSizer1411.Add(self.frequency, 1, wx.ALL, 5)

        bSizer11.Add(bSizer1411, 0, wx.EXPAND, 5)

        bSizer1412 = wx.BoxSizer(wx.HORIZONTAL)

        self.m_staticText1012 = wx.StaticText(self.general, wx.ID_ANY, _(u"Default file extension"), wx.DefaultPosition,
                                                                                    wx.DefaultSize, 0)
        self.m_staticText1012.Wrap(-1)
        bSizer1412.Add(self.m_staticText1012, 0, wx.ALL, 5)

        extensionChoices = []
        self.extension = wx.Choice(self.general, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, extensionChoices, 0)
        self.extension.SetSelection(0)
        bSizer1412.Add(self.extension, 1, wx.ALL, 5)

        bSizer11.Add(bSizer1412, 0, wx.EXPAND, 5)

        bSizer14 = wx.BoxSizer(wx.HORIZONTAL)

        self.m_staticText10 = wx.StaticText(self.general, wx.ID_ANY, _(u"Language"), wx.DefaultPosition, wx.DefaultSize, 0)
        self.m_staticText10.Wrap(-1)
        bSizer14.Add(self.m_staticText10, 0, wx.ALL, 5)

        langChChoices = []
        self.langCh = wx.Choice(self.general, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, langChChoices, 0)
        self.langCh.SetSelection(0)
        bSizer14.Add(self.langCh, 1, wx.ALL, 5)

        bSizer11.Add(bSizer14, 0, wx.EXPAND, 5)

        self.general.SetSizer(bSizer11)
        self.general.Layout()
        bSizer11.Fit(self.general)
        self.notebook.AddPage(self.general, _(u"General"), True)
        self.autoAdjust = wx.Panel(self.notebook, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL)
        bSizer18 = wx.BoxSizer(wx.VERTICAL)

        self.autoRemoveBlankLines = wx.CheckBox(self.autoAdjust, wx.ID_ANY, _(u"Offer to remove blank lines"),
                                                                                        wx.DefaultPosition, wx.DefaultSize, 0)
        bSizer18.Add(self.autoRemoveBlankLines, 0, wx.ALL, 5)

        self.autoTab2Chordpro = wx.CheckBox(self.autoAdjust, wx.ID_ANY, _(u"Offer to convert songs in tab"),
                                                                                wx.DefaultPosition, wx.DefaultSize, 0)
        bSizer18.Add(self.autoTab2Chordpro, 0, wx.ALL, 5)

        self.autoAdjustEasyKey = wx.CheckBox(self.autoAdjust, wx.ID_ANY, _(u"Offer to transpose songs to simplify chords"),
                                                                                 wx.DefaultPosition, wx.DefaultSize, 0)
        bSizer18.Add(self.autoAdjustEasyKey, 0, wx.ALL, 5)

        self.simplifyPanel = wx.ScrolledWindow(self.autoAdjust, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize,
                                                                                     wx.HSCROLL | wx.SUNKEN_BORDER | wx.VSCROLL)
        self.simplifyPanel.SetScrollRate(5, 5)
        self.simplifyPanel.SetBackgroundColour(wx.SystemSettings.GetColour(wx.SYS_COLOUR_WINDOW))

        bSizer18.Add(self.simplifyPanel, 1, wx.EXPAND | wx.ALL, 5)

        self.autoAdjust.SetSizer(bSizer18)
        self.autoAdjust.Layout()
        bSizer18.Fit(self.autoAdjust)
        self.notebook.AddPage(self.autoAdjust, _(u"AutoAdjust"), False)

        bSizer10.Add(self.notebook, 1, wx.EXPAND | wx.ALL, 5)

        m_sdbSizer3 = wx.StdDialogButtonSizer()
        self.m_sdbSizer3OK = wx.Button(self, wx.ID_OK)
        m_sdbSizer3.AddButton(self.m_sdbSizer3OK)
        self.m_sdbSizer3Cancel = wx.Button(self, wx.ID_CANCEL)
        m_sdbSizer3.AddButton(self.m_sdbSizer3Cancel)
        m_sdbSizer3.Realize();

        bSizer10.Add(m_sdbSizer3, 0, wx.ALL | wx.EXPAND, 5)

        self.SetSizer(bSizer10)
        self.Layout()

        self.Centre(wx.BOTH)

        # Connect Events
        self.fontCB.Bind(wx.EVT_KILL_FOCUS, self.OnFontSelected)
        self.sizeCB.Bind(wx.EVT_COMBOBOX, self.OnFontSelected)
        self.sizeCB.Bind(wx.EVT_KILL_FOCUS, self.OnFontSelected)
        self.sizeCB.Bind(wx.EVT_TEXT_ENTER, self.OnFontSelected)
        self.m_sdbSizer3OK.Bind(wx.EVT_BUTTON, self.OnOk)

    def __del__(self):
        pass

    # Virtual event handlers, overide them in your derived class
    def OnFontSelected(self, event):
        event.Skip()

    def OnOk(self, event):
        event.Skip()





###############################################################
# Name:             songimpress.py
# Purpose:     Generate PowerPoint presentation from songs
# Author:         Luca Allulli (webmaster@roma21.it)
# Created:     2019-02-02
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:     GNU GPL v2
##############################################################

import sys

from pptx import Presentation


class SongPresentation:
    def __init__(self, template, out_filename):
        self.pres = Presentation(template)
        self.layout = self.pres.slide_layouts[3]
        self.prev = None
        self.out_filename = out_filename

    def _add_slide(self, cur, next):
        slide = self.pres.slides.add_slide(self.layout)
        ph = list(slide.placeholders)
        t1 = ph[1]
        t2 = ph[2]
        t1.text = cur
        t2.text = next

    def _add_empty_slide(self):
        self.pres.slides.add_slide(self.layout)

    def add_line(self, line):
        if self.prev is not None:
            self._add_slide(self.prev, line)
        self.prev = line

    def end_song(self):
        if self.prev is not None:
            self._add_slide(self.prev, '')
        self._add_empty_slide()
        self.prev = None

    def close(self):
        if self.prev is not None:
            self._add_slide(self.prev, '')
        self.pres.save(self.out_filename)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()


def to_presentation(lines, output_file, template_file):
    with SongPresentation(template_file, output_file) as c:
        for line in lines:
            line = line.replace('\n', '').strip()
            if line == '---':
                c.end_song()
            elif line != '':
                c.add_line(line)


if __name__ == '__main__':
    if len(sys.argv) != 4:
        print("Usage: songimpress.py SONG_FILE OUTPUT_PRESENTATION TEMPLATE_PRESENTATION")
        exit(1)

    with open(sys.argv[1]) as f:
        to_presentation(f, sys.argv[2], sys.argv[3])



###############################################################
# Name:             MyTransposeDialog.py
# Purpose:     Transposing dialog
# Author:         Luca Allulli (webmaster@roma21.it)
# Created:     2009-12-02
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:     GNU GPL v2
##############################################################

from .TransposeDialog import *
from .Transpose import *


class MyTransposeDialog(TransposeDialog):
    def __init__(self, parent, notations, text):
        TransposeDialog.__init__(self, parent)
        self.notations = notations
        self.text = text
        fn = autodetectNotation(text, notations)
        self.fk = autodetectKey(text, fn)
        for n in self.notations:
            i = self.notation.Append(n.desc)
            self.notation.SetClientData(i, n)
            if n == fn:
                self.notation.SetSelection(i)
        self.OnNotation()
            
    def OnNotation(self, event=None):
        i = self.notation.GetSelection()
        n = self.notation.GetClientData(i)
        self.fromKey.Clear()
        self.toKey.Clear()
        for k in orderedKeys:
            kn = "%s / %s" % (
                translateChord(k, dNotation=n),
                translateChord(scales[k][1][5] + "m", dNotation=n),
            )
            i = self.fromKey.Append(kn)
            self.fromKey.SetClientData(i, k)
            if chord2pos(self.fk) == chord2pos(k):
                self.fromKey.SetSelection(i)
            i = self.toKey.Append(kn)
            self.toKey.SetClientData(i, k)
        self.UpdateToKey()
        if event is not None:
            event.Skip()
            
    def UpdateToKey(self):
        try:
            s = int(self.semitones.GetValue())
        except:
            return
        self.toKey.SetSelection((self.fromKey.GetSelection() + s) % 12)
        
    def GetTransposed(self):
        return transposeChordPro(
            self.fromKey.GetClientData(self.fromKey.GetSelection()),
            self.toKey.GetClientData(self.toKey.GetSelection()),
            self.text,
            self.notation.GetClientData(self.notation.GetSelection())
        )
        
    def OnSemitones(self, evt):
        self.UpdateToKey()
        evt.Skip()
        
    def OnFromKey(self, evt):
        self.UpdateToKey()
        evt.Skip()
        
    def OnToKey(self, evt):
        g = (self.toKey.GetSelection() - self.fromKey.GetSelection()) % 12
        if g > 7:
            g = -12 + g
        self.semitones.SetValue(g)
        evt.Skip()



###############################################################
# Name:             i18n.py
# Purpose:     Support internationalization
# Author:         Luca Allulli (webmaster@roma21.it)
# Created:     2009-09-11
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:     GNU GPL v2
##############################################################

import inspect
import os
import locale

from .Globals import *


# 2-character language code of current language
current_language = None
# wxPython wx.Locale object for current language
mylocale = None
# 2-character language code of default language, set during init
defaultLang = None
# Array mapping 2-character language codes of languages supported by Songpress
# to language names. To be set during init phase, by calling init function
supportedLangs = None


"""
Client shall:
1. Call `init`
2. Call either `setLang` (if a langugage is chosen by the user)
   or `setSystemLang`
"""


def init(default_lang, supported_langs):
    global defaultLang
    global supportedLangs
    defaultLang = default_lang
    supportedLangs = supported_langs


_ = lambda x: x


def setLang(l):
    global current_language, mylocale
    current_language = l
    langid = wx.Locale.FindLanguageInfo(l).Language
    mylocale = wx.Locale(langid)
    localedir = os.path.join(glb.path, "locale")
    xrc_localedir = os.path.join(glb.path, "xrc", "locale")
    mylocale.AddCatalogLookupPathPrefix(localedir)
    mylocale.AddCatalogLookupPathPrefix(xrc_localedir)
    mylocale.AddCatalog('songpress')
    for f in os.listdir(glb.path):
        fn, ext = os.path.splitext(f)
        if ext.lower() == '.pot':
            mylocale.AddCatalog(fn)


def getLang():
    return current_language


def setSystemLang():
    l = locale.getdefaultlocale()
    if l is not None and l[0][:2] in supportedLangs:
        setLang(l[0][:2])
    else:
        setLang(defaultLang)


def localizeXrc(filename):
    if current_language != defaultLang:
        d, domain = os.path.split(filename)
        localedir = os.path.join(glb.AddPath(d), "locale")
        mylocale.AddCatalogLookupPathPrefix(localedir)
        mylocale.AddCatalog(domain)



###############################################################
# Name:                SoongBoxes.py
# Purpose:      Elements that make up a song
# Author:           Luca Allulli (webmaster@roma21.it)
# Created:      2009-02-21
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:      GNU GPL v2
##############################################################

import wx

from .SongFormat import *


class SongBox(object):
     def __init__(self, x, y, w, h):
          object.__init__(self)
          self.x = x
          self.y = y
          self.w = w
          self.h = h
          self.marginLeft = 0
          self.marginRight = 0
          self.marginTop = 0
          self.marginBottom = 0
          self.boxes = []

     def RelocateBox(self, box):
          self.w = max(self.w, box.x + box.GetTotalWidth())
          self.h = max(self.h, box.y + box.GetTotalHeight())
          
     def AddBox(self, box):
          self.boxes.append(box)
          box.parent = self
          self.RelocateBox(box)
          
     def SetMargin(self, top, right, bottom, left):
          self.marginTop = top
          self.marginRight = right
          self.marginBottom = bottom
          self.marginLeft = left
          
     def GetTotalHeight(self):
          return self.h + self.marginTop + self.marginBottom

     def GetTotalWidth(self):
          return self.w + self.marginLeft + self.marginRight

     
class SongSong(SongBox):
     def __init__(self, format):
          SongBox.__init__(self, 0, 0, 0, 0)
          self.format = format
          self.verseCount = 0
          self.chorusCount = 0
          self.labelCount = 0
          self.drawWholeSong = False


class SongBlock(SongBox):
     # types
     verse = 1
     chorus = 2
     title = 3
     subtitle = 4

     def __init__(self, type, format):
          SongBox.__init__(self, 0, 0, 0, 0)
          self.type = type
          self.verseNumber = 0
          self.format = format
          self.drawBlock = False
          self.label = None
          self.chords = []
          
     def RemoveChordBoxes(self):
          for l in self.boxes:
               l.RemoveChordBoxes()

     def GetLastLineTextHeight(self):
          if len(self.boxes) > 0:
               return self.boxes[-1].GetTextHeight()
          return 0

               
class SongLine(SongBox):
     def __init__(self):
          SongBox.__init__(self, 0, 0, 0, 0)
          self.hasChords = False
          self.chordBaseline = 0
          self.textBaseline = 0

     def AddBox(self, text):
          if text.type == text.chord:
               self.hasChords = True
          SongBox.AddBox(self, text)
          
     def RemoveChordBoxes(self):
          self.boxes = [b for b in self.boxes if b.type != b.chord]

     def GetTextHeight(self):
          tbs = [b for b in self.boxes if b.type != b.chord]
          if len(tbs) == 0:
               return self.h
          return max(tb.h for tb in tbs)

     
class SongText(SongBox):
     text = 1
     chord = 2
     comment = 3
     title = 4
     subtitle = 5
     
     def __init__(self, text, font, type, color):
          SongBox.__init__(self, 0, 0, 0, 0)
          self.text = text
          self.font = font
          self.type = type
          self.color = color



###############################################################
# Name:             MyListDialog.py
# Purpose:     Generic dialog box asking for an item in a list
# Author:         Luca Allulli (webmaster@roma21.it)
# Created:     2019-02-02
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:     GNU GPL v2
##############################################################

import wx

from .ListDialog import ListDialog


class MyListDialog(ListDialog):
    def __init__(self, parent, message, title, elements):
        """
        Show a dialog box asking to select an item from a list

        :param message: message to be displayed
        :param title: dialog title
        :param elements: list of strings: items to be chosen from
        """
        super().__init__(parent)
        self.SetTitle(title)
        self.label.SetLabel(message)
        self.elements_list = elements
        self.elements.InsertItems(elements, 0)

    def GetSelectedIndex(self):
        return self.elements.GetSelection()

    def GetSelectedString(self):
        return self.elements_list[self.elements.GetSelection()]




###############################################################
# Name:             StandardVerseNumbers.py
# Purpose:     Decorator that adds verse numbering
# Author:         Luca Allulli (webmaster@roma21.it)
# Created:     2009-03-14
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:     GNU GPL v2
##############################################################

from songpress.SongDecorator import *
from songpress.SongFormat import *
import wx

class Format(FontFormat):
    def __init__(self, sf, chorusLabel):
        FontFormat.__init__(self)
        self.face = sf.face
        self.size = sf.size
        self.bold = sf.bold
        self.italic = sf.italic
        self.underline = sf.underline
        self.leftMargin = 0.5
        self.rightMargin = 0.5
        self.leftPadding = 0.25
        self.rightPadding = 0.25
        self.topPadding = 0.1
        self.bottomPadding = 0.1
        self.chorus = FontFormat()
        self.chorus.face = sf.chorus.face
        self.chorus.size = sf.chorus.size
        self.chorus.bold = sf.chorus.bold
        self.chorus.italic = sf.chorus.italic
        self.chorus.underline = sf.chorus.underline
        self.chorus.label = chorusLabel

    def SetChorusLabel(self, label):
        self.chorus.label = label

    def GetChorusLabel(self):
        return self.chorus.label


class Decorator(SongDecorator):
    def __init__(self, format):
        SongDecorator.__init__(self)
        self.format = format

    wxBlack = wx.Colour(0, 0, 0)
    wxWhite = wx.Colour(255, 255, 255)
    wxGrey = wx.Colour(200, 200, 200)

    def SetMarginBlock(self, block):
        font = block.format.wxFont
        self.dc.SetFont(font)
        baseWidth, baseHeight = self.dc.GetTextExtent("0")
        if block.type == block.verse:
            text = block.label if block.label is not None else ''
            text_spacer = text if text != '' else str(self.s.labelCount)
            w, h = self.dc.GetTextExtent(text_spacer)

            w += baseWidth * (
                self.format.leftMargin + self.format.leftPadding + self.format.rightMargin + self.format.rightPadding
            )
        elif block.type == block.title:
            w = 0
        else:
            if block.label is not None:
                text = block.label
            else:
                text = self.format.chorus.label
            w, h = self.dc.GetTextExtent(text)
            w += baseWidth * (
                self.format.leftMargin + self.format.leftPadding + self.format.rightMargin + self.format.rightPadding
            )
        block.SetMargin(0, 0, 0, w)

    def PreDrawBlock(self, block, bx, by):
        if block.type != block.title and len(block.boxes) > 0:
            font = block.format.wxFont
            self.dc.SetFont(font)
            baseWidth, baseHeight = self.dc.GetTextExtent("0")
            if block.type == block.verse:
                background = self.wxGrey
                foreground = self.wxBlack
                if block.label is not None:
                    text = block.label
                    text_spacer = text
                    if text == '':
                        text_spacer = str(self.s.labelCount)
                else:
                    text = str(block.verseLabelNumber)
                    text_spacer = str(self.s.labelCount)
                w, h = self.dc.GetTextExtent(text_spacer)

            else:
                background = self.wxBlack
                foreground = self.wxWhite
                if block.label is not None:
                    text = block.label
                else:
                    text = self.format.chorus.label
                w, h = self.dc.GetTextExtent(text)
            if text != '':
                realW, realH = self.dc.GetTextExtent(text)
                rx = bx + self.format.leftMargin * baseWidth
                tx = (rx
                    + baseWidth * self.format.leftPadding
                    + 0.5 * (w - realW))
                ty = by + block.boxes[0].textBaseline + block.marginTop - h
                ry = ty - self.format.topPadding * baseHeight
                brush = wx.Brush(background, wx.SOLID)
                self.dc.SetBrush(brush)
                self.dc.DrawRectangle(int(rx), int(ry),
                    int(w + baseWidth * (self.format.leftPadding + self.format.rightPadding)),
                    int(h + baseHeight * (self.format.topPadding + self.format.bottomPadding)))
                brush = wx.Brush(foreground, wx.SOLID)
                self.dc.SetBrush(brush)
                self.dc.SetTextForeground(foreground)
                self.dc.SetBackgroundMode(wx.TRANSPARENT)
                self.dc.DrawText(text, int(tx), int(ty))
                self.dc.SetTextForeground(self.wxBlack)



# -*- coding: utf-8 -*- 

import wx
import wx.html

_ = wx.GetTranslation


# -*- coding: utf-8 -*- 

###########################################################################
## Python code generated with wxFormBuilder (version Sep  8 2010)
## http://www.wxformbuilder.org/
##
## PLEASE DO "NOT" EDIT THIS FILE!
###########################################################################


###########################################################################
## Class UpdatePanel
###########################################################################

class UpdatePanel ( wx.Panel ):
    
    def __init__( self, parent ):
        wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( -1,-1 ), style = wx.TAB_TRAVERSAL )
        
        bSizer17 = wx.BoxSizer( wx.VERTICAL )
        
        fgSizer1 = wx.FlexGridSizer( 2, 2, 0, 0 )
        fgSizer1.AddGrowableCol( 1 )
        fgSizer1.AddGrowableRow( 1 )
        fgSizer1.SetFlexibleDirection( wx.BOTH )
        fgSizer1.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
        
        self.m_staticText12 = wx.StaticText( self, wx.ID_ANY, _("Version number"), wx.DefaultPosition, wx.DefaultSize, 0 )
        self.m_staticText12.Wrap( -1 )
        fgSizer1.Add( self.m_staticText12, 0, wx.ALL, 5 )
        
        self.version = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_READONLY )
        fgSizer1.Add( self.version, 0, wx.ALL|wx.EXPAND, 5 )
        
        self.m_staticText13 = wx.StaticText( self, wx.ID_ANY, _("What's new"), wx.DefaultPosition, wx.DefaultSize, 0 )
        self.m_staticText13.Wrap( -1 )
        fgSizer1.Add( self.m_staticText13, 0, wx.ALL, 5 )
        
        self.new_features = wx.html.HtmlWindow( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.html.HW_SCROLLBAR_AUTO )
        fgSizer1.Add( self.new_features, 1, wx.ALL|wx.EXPAND, 5 )
        
        bSizer17.Add( fgSizer1, 1, wx.EXPAND, 5 )
        
        bSizer18 = wx.BoxSizer( wx.HORIZONTAL )
        
        
        bSizer18.Add( ( 0, 0), 1, wx.EXPAND, 5 )
        
        self.download = wx.Button( self, wx.ID_ANY, _("Download"), wx.DefaultPosition, wx.DefaultSize, 0 )
        bSizer18.Add( self.download, 0, wx.ALL, 5 )
        
        self.remind = wx.Button( self, wx.ID_ANY, _("Remind later"), wx.DefaultPosition, wx.DefaultSize, 0 )
        bSizer18.Add( self.remind, 0, wx.ALL, 5 )
        
        self.skip = wx.Button( self, wx.ID_ANY, _("Skip this version"), wx.DefaultPosition, wx.DefaultSize, 0 )
        bSizer18.Add( self.skip, 0, wx.ALL, 5 )
        
        bSizer17.Add( bSizer18, 0, wx.EXPAND, 5 )
        
        self.SetSizer( bSizer17 )
        self.Layout()
        bSizer17.Fit( self )
        
        # Connect Events
        self.download.Bind( wx.EVT_BUTTON, self.OnDownload )
        self.remind.Bind( wx.EVT_BUTTON, self.OnRemind )
        self.skip.Bind( wx.EVT_BUTTON, self.OnSkip )
    
    def __del__( self ):
        pass
    
    
    # Virtual event handlers, overide them in your derived class
    def OnDownload( self, event ):
        event.Skip()
    
    def OnRemind( self, event ):
        event.Skip()
    
    def OnSkip( self, event ):
        event.Skip()
    




###############################################################
# Name:             SongTokenizer.py
# Purpose:     Subclass of Tokenizer to tokenize songs
# Author:         Luca Allulli (webmaster@roma21.it)
# Created:     2009-01-31
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:     GNU GPL v2
##############################################################

        
import re

from .Tokenizer import *


class SongTokenizer(Tokenizer):
    def __init__(self, l):
        Tokenizer.__init__(self, l)
        
    #Tokens
    openCurlyToken = TokenType('(\{)', 'openCurlyToken')
    closeCurlyToken = TokenType('(\})', 'closeCurlyToken')
    commentToken = TokenType('^([ \t]*#.*)$', 'commentToken')
    normalToken = TokenType('([^[{]+)', 'normalToken')
    commandToken = TokenType('([^}:]+)', 'commandToken')
    attrToken = TokenType('([^}]+)', 'attrToken')
    chordToken = TokenType('(\[[^]]*)', 'chordToken')
    closeChordToken = TokenType('(\])', 'closeChordToken')
    colonToken = TokenType('(:)', 'colonToken')

    #States
    normal = 1
    command = 2
    attr = 3
    
    #Transition function
    transition = {
        normal: (
            (openCurlyToken, command),
            (chordToken, normal),
            (closeChordToken, normal),
            (commentToken, normal),
            (normalToken, normal),
        ),
        command: (
            (colonToken, attr),
            (closeCurlyToken, normal),
            (commandToken, command),
        ),
        attr: (
            (attrToken, attr),
            (closeCurlyToken, normal),
            (commentToken, normal),
        )
    }




###############################################################
# Name:             HTML.py
# Purpose:     Export song to HTML
# Author:         Luca Allulli (webmaster@roma21.it)
# Created:     2010-05-05
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:     GNU GPL v2
##############################################################

from .Renderer import *

template = """
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<head>
        <title>%(title)s</title>
        <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
        <style type="text/css">
            <!--
                div.chorus,div.verse,div.title,div.titleblock {
                    margin-bottom: 1.5em;
                    font-family: "%(face)s";
                }
                div.chorus table,div.verse table, div.title table {
                    margin: 0px;
                    padding: 0px;
                    border: 0px;
                    border-collapse: collapse;
                }
                .chorus {
                    font-weight: bold;
                }
                .chord {
                    font-style: italic;
                    font-size: small;
                    font-weight: normal;
                }
                .title {
                    font-weight: bold;
                    text-decoration: underline;
                }
                .subtitle {
                    font-weight: normal;
                    font-style: italic;
                    font-size: 0.95em;
                }
            -->
        </style>
    </head>
    <body>
        %(body)s
    </body>
</html>
"""

class HtmlExporter(object):
    def __init__(self, sf):
        object.__init__(self)
        self.sf = sf
        self.out = ""

    def Draw(self, song, dc):
        # dc is a dummy parameter
        classes = {
            SongBlock.title: 'titleblock',
            SongBlock.chorus: 'chorus',
            SongBlock.verse: 'verse',
        }
        out = ""
        title = "Songpress"
        for block in song.boxes:
            b = "<div class=\"%s\">" % (classes[block.type])
            for line in block.boxes:
                tc = "<td>"
                tt = "<td>"
                new_line = True
                for t in line.boxes:
                    if t.type == SongText.title:
                        tt += f"""<span class="title">{t.text.replace(" ", "&nbsp;")}</span>"""
                    elif t.type == SongText.subtitle:
                        tt += f"""<span class="subtitle">{t.text.replace(" ", "&nbsp;")}</span>"""
                    elif t.type == SongText.chord:
                        if not new_line:
                            tc += "</td><td>"
                            tt += "</td><td>"
                        tc += t.text.replace(" ", "&nbsp;")
                    else:
                        tt += t.text.replace(" ", "&nbsp;")
                        if block.type == SongBlock.title:
                            title = t.text
                    new_line = False
                tc += "</td>"
                tt += "</td>"
                b += "<table cellpadding=\"0\">\n<tr class=\"chord\">%s</tr>\n<tr>%s</tr>\n</table>\n" % (tc, tt)
            b += "</div>"
            out += b
        self.out = template % {
                'title': title,
                'body': out,
                'face': self.sf.face,
            }
        return 0, 0

    def getHtml(self):
        return self.out


class TabExporter(object):
    def __init__(self, sf):
        object.__init__(self)
        self.sf = sf
        self.out = ""

    def Draw(self, song, dc):
        # dc is a dummy parameter
        out = []
        for block in song.boxes:
            for line in block.boxes:
                tc = ""
                tt = ""
                spaces = 0
                for t in line.boxes:
                    if t.type == SongText.chord:
                        # n = spaces + t.text
                        if spaces >= 0:
                            tc += " " * spaces
                        else:
                            tt += " " * -spaces
                        tc += t.text
                        spaces = -len(t.text)
                    else:
                        tt += t.text
                        spaces += len(t.text)
                if tc != "":
                    out.append(tc)
                out.append(tt)
            out.append("\n")
        self.out = "\n".join(out).strip()
        return 0, 0

    def getTab(self):
        return self.out



###############################################################
# Name:             MyNotationDialog.py
# Purpose:     Dialog to change notation
# Author:         Luca Allulli (webmaster@roma21.it)
# Created:     2009-12-09
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:     GNU GPL v2
##############################################################

from .NotationDialog import *
from .Transpose import *


class MyNotationDialog(NotationDialog):
    def __init__(self, parent, notations, text):
        NotationDialog.__init__(self, parent)
        self.notations = notations
        fn = autodetectNotation(text, notations)
        for n in self.notations:
            i = self.fromNotation.Append(n.desc)
            self.fromNotation.SetClientData(i, n)
            if n == fn:
                self.fromNotation.SetSelection(i)
            i = self.toNotation.Append(n.desc)
            self.toNotation.SetClientData(i, n)
        self.toNotation.SetSelection(0)
        self.OnFromNotation(None)
        self.text = text
            
    def OnFromNotation(self, event):
        i = self.fromNotation.GetSelection()
        if self.toNotation.GetSelection() == i:
            self.toNotation.SetSelection(1 if i == 0 else 0) 
        
    def ChangeChordNotation(self):
        return translateChordPro(
            self.text,
            self.fromNotation.GetClientData(self.fromNotation.GetSelection()),
            self.toNotation.GetClientData(self.toNotation.GetSelection())
        )



###############################################################
# Name:             Pref.py
# Purpose:     Preference management
# Author:         Luca Allulli (webmaster@roma21.it)
# Created:     2009-05-01
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:     GNU GPL v2
##############################################################

import copy
import string
from xml.dom import minidom


class Property(object):
    """Structure for properties"""
    
    def __getValue(self):
        return self.__value
        
    def __setValue(self, value):
        if self.validator != None:
            valid, value = self.validator(value)
            if valid:
                self.__value = value
        else:
            self.__value = value
        if self.gui != None:
            self.gui.Changed()

    value = property(__getValue, __setValue)
    
    def __init__(self, value, validator = None, gui = None):
        object.__init__(self)
        self.validator = validator
        self.gui = gui(self) if gui != None else None
        self.value = value
    
        
class PropertyDef(object):
    """Structure for property definitions"""
    def __init__(self, name, ptype, init = None, validator = None, gui = None):
        object.__init__(self)
        self.name = name
        self.ptype = ptype
        self.init = init
        self.validator = validator
        self.gui = gui


class XmlSerializer(object):
    def __init__(self):
        object.__init__(self)
        self.idn = 0
        self.dom = minidom.Document()
        self.ids = {}
        
    def RegisterId(self, p):
        self.idn += 1
        self.ids[p] = self.idn
        return str(self.idn)
        
    def GetId(self, p):
        return str(self.ids[p])
        
    def Serialize(self, p):
        self.dom.appendChild(p.XmlSerialize(self))
                
class XmlDeserializer(object):
    def __init__(self, dom, classes):
        object.__init__(self)
        self.dom = dom
        self.ids = {}
        self.classes = dict((c.__name__, c) for c in classes)
        
    def RegisterId(self, id, p):
        self.ids[id] = p
        
    def GetById(self, p):
        return self.ids[p]
        
    def Serialize(self, p):
        self.dom.appendChild(p.XmlSerialize(self))
        
    def GetClass(self, name):
        return self.classes[name]
        
    def Deserialize(self):
        l = []
        for el in self.dom.childNodes:
            l.append(Preferences.XmlDeserialize(self, el))
        return l
        
        
class NameNotFoundException(Exception):
    def __init__(self, arg):
        Exception.__init__(self, arg)


class Preferences(object):
    """Class for structured property objects."""
    def __init__(self, parents = [], gui = None):
        self.__prefs = {}
        object.__init__(self)
        self.__parents = dict((x, x) for x in parents)
        self.__parentsByName = dict((x, x) for x in parents)
        self.__owner = None
        self.__CacheParents()
        self.__gui = gui
        for n, v in self.__GetAllPrefDef():
            f = v.init
            if f != None:
                self.__setattr__(n, f())
        
    prefDef = {}
    
    @classmethod
    def Register(cls, name, ptype, init = None, validator = None, gui = None):
        if not ('prefDef' in cls.__dict__):
            cls.prefDef = {}
        cls.prefDef[name] = PropertyDef(name, ptype, init, validator, gui)
        
    def __SetOwner(self, owner):
        self.__owner = owner
        self.__CacheParents()
        
    def __CacheParents(self):
        def Navigate(node, path):
            if node == None:
                return None
            if path == '..':
                return node.__owner
            if path in node.__prefs:
                return node.__prefs.value
            return None
        for n in self.__parentsByName:
            if type(n) == str:
                del self.__parents[self.__parentsByName[n]]
                val = reduce(Navigate, string.split(n, '/'), self)
                self.__parentsByName[n] = val
                self.__parents[val] = n        
        
    def __GetPrefByName(self, name):
        if name in self.__prefs:
            return self.__prefs[name]
        for p in self.__parents:
            try:
                return p.__GetPrefByName(name)
            except NameNotFoundException:
                pass
        raise NameNotFoundException(name)
    
    @classmethod
    def __GetPrefDefByName(cls, name):
        if name in cls.prefDef:
            return cls.prefDef[name]
        for b in cls.__bases__:
            try:
                return b.__GetPrefDefByName(name)
            except Exception:
                pass
        raise NameNotFoundException(name)
    
    def __getattr__(self, name):
        return self.__GetPrefByName(name).value
        
    def __setattr__(self, name, value):
        if name == '_Preferences__prefs':
            object.__setattr__(self, name, value)
            return
        try:
            pd = self.__GetPrefDefByName(name)
            assert isinstance(value, pd.ptype)
            if not name in self.__prefs:
                self.__prefs[name] = Property(value, pd.validator, pd.gui)
            else:
                self.__prefs[name].value = value
            if isinstance(value, Preferences):
                value.__SetOwner(self)
        except NameNotFoundException:
            object.__setattr__(self, name, value)
    
    def GetInherit(self, name):
        return not (name in self.__prefs)
        
    def SetInherit(self, name, inherit):
        if inherit:
            if name in self.__prefs:
                del self.__prefs[name]
        else:
            p = self.__GetPrefByName(name)
            self.__prefs[name] = Preferences.__DeepCopyPrefProp(p, None, self)
            
    def __GetAllPrefs(self):
        for n in self.__prefs:
            yield n, self.__prefs[n]
        for p in self.__parents:
            for n, v in p.__GetAllPrefs():
                yield n, v
    
    @classmethod
    def __GetAllPrefDef(cls):
        if 'prefDef' in cls.__dict__:    
            for n in cls.prefDef:
                yield n, cls.prefDef[n]
        for b in cls.__bases__:
            if isinstance(b, Preferences):
                for n, v in b.__GetAllPrefDef():
                    yield n, v
    
    @staticmethod
    def __DeepCopyPrefProp(p, d, owner):
        q = Property(
            Preferences.__DeepCopyPref(p.value, d, owner) if isinstance(p.value, Preferences) else copy.deepcopy(p.value, d),
            copy.deepcopy(p.validator, d),
            copy.deepcopy(p.gui, d)
        )
        return q
    
    @staticmethod 
    def __DeepCopyPref(p, d, owner):
        c = Preferences(
            [pa for pa in p.__parentsByName],
            copy.deepcopy(p.__gui, d)
        )
        c.__owner = owner
        c.__CacheParents()
        for n in p.__prefs:
            c.__prefs[n] = Preferences.__DeepCopyPrefProp(p.__prefs[n], p)
        return c
        
    def XmlSerialize(self, xm):
        xd = xm.dom
        pref = xd.createElement("pref")
        pref.setAttribute("ptype", self.__class__.__name__)
        pref.setAttribute("id", xm.RegisterId(self))
        for n in self.__parentsByName:
            par = xd.createElement("parent")
            if type(n) == str:
                par.setAttribute("rel", n)
            else:
                par.setAttribute("id", xm.GetId(n))
            pref.appendChild(par)
        for n in self.__prefs:
            elem = xd.createElement("elem")
            elem.setAttribute("name", n)
            p = self.__prefs[n].value
            if isinstance(p, Preferences):
                elem.appendChild(p.XmlSerialize(xm))
            else:
                t = xd.createElement(type(p).__name__)
                t.appendChild(xd.createTextNode(str(p)))
                elem.appendChild(t)
            pref.appendChild(elem)
        return pref
        
    @staticmethod
    def XmlDeserialize(xm, pref):
        while pref.nodeName == '#text':
            pref = pref.nextSibling
        if pref.nodeName == 'pref':
            p = [c.getAttribute('rel') if c.hasAttribute('rel') else xm.GetById(c.getAttribute('id')) for c in pref.childNodes if c.nodeName == 'parent']
            c = xm.GetClass(pref.getAttribute('ptype'))
            assert issubclass(c, Preferences)
            pr = c(p)
            xm.RegisterId(pref.getAttribute('id'), pr)
            for c in pref.childNodes:
                if c.nodeName == 'elem':
                    name = c.getAttribute('name')
                    pr.__setattr__(name, pr.XmlDeserialize(xm, c.firstChild))
            return pr
        elif pref.nodeName == 'str':
            return str(pref.firstChild.nodeValue)
        elif pref.nodeName == 'int':
            return int(pref.firstChild.nodeValue)
            
            




from pyshortcuts import make_shortcut
from pathlib import Path
import sys
import os


def get_executable_path():
    exe = Path(sys.argv[0]).resolve()
    return str(exe)


def get_current_dir():
    return os.path.dirname(os.path.abspath(__file__))


def create_shortcuts():
    icon = os.path.join(get_current_dir(), 'img/songpress.ico')
    print("Icon: ", icon)
    make_shortcut(
        get_executable_path(),
        "Songpress",
        icon=icon,
        desktop=False,
        noexe=True,
    )



# -*- coding: iso-8859-1 -*-

###############################################################
# Name:             Transpose.py
# Purpose:     Transposing services
# Author:         Luca Allulli (webmaster@roma21.it)
# Created:     2009-11-18
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:     GNU GPL v2
##############################################################

import re
import math

import wx


_ = wx.GetTranslation


class Notation(object):
    def __init__(self, id, desc, chords, repl, replrev):
        object.__init__(self)
        self.id = id
        self.descv = desc
        self.chords = chords
        self.chordDict = {}
        i = 0
        for k in chords:
            self.chordDict[k.upper()] = i
            i += 1
        self.repl = [(re.compile(x[0]), x[1]) for x in repl]
        self.replrev = [(re.compile(x[0]), x[1]) for x in replrev]

    def GetDesc(self):
        return _(self.descv)

    def SetDesc(self, v):
        pass

    desc = property(GetDesc, SetDesc)

    def Ord2Chord(self, pos):
        return self.chords[pos]

    def Chord2Ord(self, chord):
        return self.chordDict[chord.upper()]

    def __AlterationStandard(self, a, rs):
        for r in rs:
            p = 0
            b = ''
            for m in r[0].finditer(a):
                b += a[p:m.start()] + r[1]
                p = m.end()
            b += a[p:]
            a = b
        return a

    def PostprocessingFromStandard(self, c, a):
        return c, a

    def PreprocessingToStandard(self, chord):
        return chord

    def AlterationFromStandard(self, a):
        return self.__AlterationStandard(a, self.repl)

    def AlterationToStandard(self, a):
        return self.__AlterationStandard(a, self.replrev)


enNotation = Notation(
    "enNotation",
    _("American (C D E... B)"),
    ['C', 'D', 'E', 'F', 'G', 'A', 'B'],
    [],
    []
)

itNotation = Notation(
    "itNotation",
    _("Italian (Do Re Mi... Si)"),
    ['Do', 'Re', 'Mi', 'Fa', 'Sol', 'La', 'Si'],
    [
        (r'maj7', '7+'),
        (r'sus4', '4'),
        (r'^m', '-')
    ],
    [
        (r'7\+', 'maj7'),
        (r'^4', 'sus4'),
        (r'^-', 'm')
    ]
)


itUcNotation = Notation(
    "itUcNotation",
    _("Uppercase Italian (DO RE MI... SI)"),
    ['DO', 'RE', 'MI', 'FA', 'SOL', 'LA', 'SI'],
    [
        (r'maj7', '7+'),
        (r'sus4', '4'),
        (r'^m', '-')
    ],
    [
        (r'7\+', 'maj7'),
        (r'^4', 'sus4'),
        (r'^-', 'm')
    ]
)

frNotation = Notation(
    "frNotation",
    _(u"French (Do Ré Mi... Si)"),
    ['Do', u'Ré', 'Mi', 'Fa', 'Sol', 'La', 'Si'],
    [
        (r'maj7', '7+'),
        (r'sus4', '4'),
    ],
    [
        (r'7\+', 'maj7'),
        (r'^4', 'sus4'),
    ]
)

ptNotation = Notation(
    "ptNotation",
    _(u"Portuguese (Dó Ré Mi... Si)"),
    [u'Dó', u'Ré', 'Mi', u'Fá', 'Sol', u'Lá', 'Si'],
    [
        (r'maj7', '7+'),
        (r'sus4', '4'),
        (r'^m', '-')
    ],
    [
        (r'7\+', 'maj7'),
        (r'^4', 'sus4'),
        (r'^-', 'm')
    ]
)


easyChords = {
    'basic': (_("Basic chords (A, E, D)"), ["A", "E", "D"], 4),
    'Cprog': (_("50s progr. in C (C, Am, Dm, G7)"), ["C", "Am", "Dm", "G7"], 4),
    'F': (_("F chord"), ["F"], 2),
    'Gprog': (_("50s progr. in G (G, Em, Am, D7)"), ["G", "Em", "Am", "D7"], 4),
    'Dprog': (_("50s progr. in D (D, Bm, Em, A7)"), ["D", "Bm", "Em", "A7"], 1),
    'Aprog': (_("50s progr. in A (A, F#m, Bm, E7)"), ["A", "F#m", "Bm", "E7"], 1),
    'C#(m)(7)': (_("C#, C#m, C#7 chords"), ["C#", "C#m", "C#7"], 0),
    'Fprog': (_("50s progr. in F (F, Dm, Gm, C7)"), ["F", "Dm", "Gm", "C7"], 0),
    'BB7': (_("B and B7 chords"), ["B", "B7"], 0),
}

def getEasyChordsDescription(e):
    return _(e[0])

easyChordsOrder = ['basic', 'Cprog', 'F', 'Gprog', 'Dprog', 'Aprog', 'C#(m)(7)', 'Fprog', 'BB7']


class GermanNotation(Notation):
    def PreprocessingToStandard(self, chord):
        if chord.lower().startswith('bb'):
            return "Hb" + chord[2:]
        return chord

    def PostprocessingFromStandard(self, c, a):
        if c == 'Hb':
            c = 'Bb'
        return c, a


trad_de_map = [
    ('C#', 'Cis'),
    ('Db', 'Des'),
    ('Eb', 'Es'),
    ('F#', 'Fis'),
    ('Gb', 'Ges'),
    ('G#', 'Gis'),
    ('Ab', 'As'),
    ('A#', 'Ais'),
    ('Hb', 'B'),
]

trad_de_from = {a.lower(): b for a, b in trad_de_map}
trad_de_to = [(b.lower(), a) for a, b in trad_de_map]


class TraditionalGermanNotation(Notation):
    def PreprocessingToStandard(self, chord):
        cl = chord.lower()
        for a, b in trad_de_to:
            if cl.startswith(a):
                return b + chord[len(a):]
        return chord

    def PostprocessingFromStandard(self, c, a):
        c = trad_de_from.get(c.lower(), c)
        if a != "" and a[0] == 'm' and not a.startswith('maj'):
            c = c.lower()
        return c, a


deNotation = GermanNotation(
    "deNotation",
    _("German/Sacandinavian (C C#/Db D... Bb H)"),
    ['C', 'D', 'E', 'F', 'G', 'A', 'H'],
    [],
    []
)


tradDeNotation = TraditionalGermanNotation(
    "tradDeNotation",
    _("Traditional German (C Cis/Des D... B H)"),
    ['C', 'D', 'E', 'F', 'G', 'A', 'H'],
    [
        ('C#', 'Cis'),
        ('Db', 'Des'),
    ],
    [
        ('C#', 'Cis'),
        ('Db', 'Des'),
    ]
)


defaultLangNotation = {
    'en': enNotation,
    'it': itNotation,
    'fr': frNotation,
    'de': deNotation,
}


naturalScale = ['C', 'C#', 'D', 'D#', 'E', 'F', 'F#', 'G', 'G#', 'A', 'A#', 'B']

tone = {
    'C': 0,
    'D': 2,
    'E': 4,
    'F': 5,
    'G': 7,
    'A': 9,
    'B': 11
}

interval = [
 (0, 0),
 (0, 1),
 (1, 0),
 (1, 1),
 (2, 0),
 (3, 0),
 (3, 1),
 (4, 0),
 (4, 1),
 (5, 0),
 (5, 1),
 (6, 0),
 (6, 1),
 (7, 0)
]

scales = {
    'C': (0, ['C', 'D', 'E', 'F', 'G', 'A', 'B']),
    'Db': (1, ['Db', 'Eb', 'F', 'Gb', 'Ab', 'Bb', 'C']),
    'D': (2, ['D', 'E', 'F#', 'G', 'A', 'B', 'C#']),
    'Eb': (3, ['Eb', 'F', 'G', 'Ab', 'Bb', 'C', 'D']),
    'E': (4, ['E', 'F#', 'G#', 'A', 'B', 'C#', 'D#']),
    'F': (5, ['F', 'G', 'A', 'Bb', 'C', 'D', 'E']),
    'Gb': (6, ['Gb', 'Ab', 'Bb', 'Cb', 'Db', 'Eb', 'F']),
    'G': (7, ['G', 'A', 'B', 'C', 'D', 'E', 'F#']),
    'Ab': (8, ['Ab', 'Bb', 'C', 'Db', 'Eb', 'F', 'G']),
    'A': (9, ['A', 'B', 'C#', 'D', 'E', 'F#', 'G#']),
    'Bb': (10, ['Bb', 'C', 'D', 'Eb', 'F', 'G', 'A']),
    'B': (11, ['B', 'C#', 'D#', 'E', 'F#', 'G#', 'A#'])
}
orderedKeys = ['C', 'Db', 'D', 'Eb', 'E', 'F', 'Gb', 'G', 'Ab', 'A', 'Bb', 'B']

vectorModes = ['', 'm', '7', 'm7']

referenceVector = [0.65713162540630443, 0.0, 0.037841466800806009, 0.0, 0.0, 0.0014554410308002313, 0.0, 0.0, 0.026197938554404162, 0.12444020813341977, 0.0014554410308002313, 0.029108820616004623, 0.0, 0.0, 0.0, 0.0, 0.040024628347006361, 0.12589564916422, 0.030564261646804855, 0.0036386025770005779, 0.46210252727907342, 0.013098969277202081, 0.0, 0.0, 0.0, 0.00072772051540011566, 0.0, 0.0, 0.49703311201827893, 0.0, 0.016009851338802544, 0.0, 0.00072772051540011566, 0.0, 0.0, 0.0, 0.0094603667002015022, 0.2648902676056421, 0.0058217641232009253, 0.0043663230924006939, 0.0014554410308002313, 0.0, 0.00072772051540011566, 0.0, 0.0, 0.0014554410308002313, 0.0, 0.0]


def splitChord(c, locNotation=enNotation):
    for k in locNotation.chords:
        if c.upper().startswith(k.upper()):
            if len(c) != len(k):
                d = c[len(k)]
                if d == "b" or d == "#":
                    return (k+d, c[(len(k) + 1):])
                return (k, c[len(k):])
            return (c, "")
    return ("", c)


def __alteration(chord):
    if len(chord) == 1:
        return (chord, 0)
    elif chord[1] == '#':
        return (chord[0], 1)
    else:
        return (chord[0], -1)


def chord2pos(chord, key="C"):
    c, a = __alteration(chord)
    s, b = __alteration(key)
    return (tone[c.upper()] + a - tone[s.upper()] - b) % 12


def __pos2chord(pos, key):
    n, i = interval[pos]
    # if pos in scale, use it
    if i == 0:
        return scales[key][1][n]
    # else use natural scale
    ref = scales[key][0]
    diff = (pos + ref) % 12
    return naturalScale[diff]


def transpose(s, d, chord, notation=enNotation):
    sl = chord.find("/")
    if sl > -1:
        return "%s/%s" % (transpose(s, d, chord[:sl], notation), transpose(s, d, chord[sl + 1:], notation))
    chord = translateChord(chord, notation, enNotation)
    c, v = splitChord(chord)
    if c == "":
        return chord
    p = chord2pos(c, s)
    return translateChord(__pos2chord(p, d) + v, enNotation, notation)


def translateChord(chord, sNotation=enNotation, dNotation=enNotation):
    #if sNotation == dNotation:
    #    return chord
    sl = chord.find("/")
    if sl > -1:
        return "%s/%s" % (translateChord(chord[:sl], sNotation, dNotation), translateChord(chord[sl + 1:], sNotation, dNotation))
    chord = sNotation.PreprocessingToStandard(chord)
    c, a = splitChord(chord, sNotation)
    if c == "":
        return chord
    alt = c[-1]
    if alt == '#' or alt == 'b':
        c = c[:-1]
    else:
        alt = ""
    if c == "":
        return chord
    d = dNotation.Ord2Chord(sNotation.Chord2Ord(c)) + alt
    b = dNotation.AlterationFromStandard(sNotation.AlterationToStandard(a))
    d, b = dNotation.PostprocessingFromStandard(d, b)
    return d + b


def transposeChordPro(s, d, text, notation=enNotation):
    r = re.compile('\[([^]]*)\]')
    p = 0
    b = ''
    for m in r.finditer(text):
        b += "%s[%s]" % (
            text[p:m.start()],
            transpose(s, d, m.group(1), notation)
        )
        p = m.end()
    return b + text[p:]


def translateChordPro(text, sNotation=enNotation, dNotation=enNotation):
    r = re.compile('\[([^]]*)\]')
    p = 0
    b = ''
    for m in r.finditer(text):
        b += "%s[%s]" % (
            text[p:m.start()],
            translateChord(m.group(1), sNotation, dNotation)
        )
        p = m.end()
    return b + text[p:]


def autodetectNotation(text, notations):
    r = re.compile('\[([^]]*)\]')
    cnt = [0 for x in notations]
    for m in r.finditer(text):
        for i in range(0, len(notations)):
            c, a = splitChord(m.group(1), notations[i])
            if c != "":
                cnt[i] += 1
    return notations[cnt.index(max(cnt))]


def normalize(vector):
    count = 0
    for c in vector:
        count += c**2
    if count != 0:
        count = math.sqrt(count)
        return [x/count for x in vector]
    else:
        return vector


def scalarProduct(v1, v2):
    count = 0
    for i in range(0, len(v1)):
        count += v1[i]*v2[i]
    return count


def vectorizeChords(text, notation=enNotation):
    r = re.compile('\[([^]]*)\]')
    v = [0 for x in range(12 * len(vectorModes))]
    for m in r.finditer(text):
        chord = translateChord(m.group(1), notation, enNotation)
        c, a = splitChord(chord)
        if c != "" and a in vectorModes:
            v[chord2pos(c, "C") * len(vectorModes) + vectorModes.index(a)] += 1
    return normalize(v)


def autodetectKey(text, notation=enNotation):
    v = vectorizeChords(text, notation)
    r = referenceVector
    max = 0
    key = 0
    n = len(vectorModes)
    for k in range(0, 12):
        s = scalarProduct(v, r)
        if s > max:
            max = s
            key = k
        r = r[-n:] + r[:-n]
    return orderedKeys[key]


def integrateChords(chords, text):
    """Integrate chord line in text line, as chordpro"""
    r = re.compile(r'(\S+)')
    i = r.finditer(chords)
    l = [x for x in i]
    l.reverse()
    if l != []:
        lc = l[0].start()
        lt = len(text)
        for i in range(0, lc-lt):
            text += ' '
        for m in l:
            s = m.start()
            text = text[:s] + '[' + m.group() + ']' + text[s:]
    return text


def testChordLine(line, notation=enNotation):
    """Return True iff line contains only chords"""
    # First, tokenize line: if a token is not a chord => False
    r = re.compile(r'(\S+)')
    l = 0
    n = 0
    for m in r.finditer(line):
        c = m.group()
        c = notation.PreprocessingToStandard(c)
        e, a = splitChord(c, notation)
        if e == '':
            return False
        l += len(translateChord(c, notation, enNotation))
        n += 1
    # We don't consider an empty line a chord line
    if n == 0:
        return False
    # Looks okay... if a whitespace is occurring twice => True
    if line.find("  ") != -1:
        return True
    # Uhm, weird... are chord tokens "short"? Yes <=> True
    return l/float(n) <= 2.3


def testTabFormat(text, notations):
    """
    Test whether text is in tab format

    It happens iff at least 3 lines are chord lines.
    Return the best matching notation, or None
    notations is a list of notations to be tested
    """
    lines = text.splitlines()
    max = 0
    maxn = None
    for n in notations:
        nc = 0
        for l in lines:
            if testChordLine(l, n):
                nc += 1
        if nc > max and nc >= 3:
            max = nc
            maxn = n
    return maxn


def tab2ChordPro(text, notation=enNotation):
    """
    Convert text from tab to chordpro format
    """
    l = text.splitlines()
    n = len(l)
    i = 0
    out = []
    while i < n:
        if i + 1 < n and testChordLine(l[i], notation) and not testChordLine(l[i+1], notation):
            out.append(integrateChords(l[i], l[i+1]))
            i += 2
        else:
            out.append(l[i])
            i += 1
    return "\n".join(out)


def testSpuriousLines(text):
    """
    Determine if there are spurious empty lines

    It happens iff at least 1/3 of the lines are empty,
    and there are at least 3 non-empty lines
    """
    c = 0
    ll = text.splitlines()
    for l in ll:
        if l.strip() == '':
            c += 1
    return c >= len(ll)/3.0 and len(ll) - c >= 3


def removeSpuriousLines(text):
    """
    Remove spurious blank lines from text

    If a blank line is isolated, remove it
    If there are several contiguous blank lines, keep only one of them
    """
    out = []
    ll = text.splitlines()
    ln = len(ll)
    i = 0
    def empty(l):
        return l.strip() == ''
    while i < ln:
        if empty(ll[i]):
            if i + 1 < ln and empty(ll[i + 1]):
                out.append('')
                i += 2
                while i < ln and empty(ll[i]):
                    i += 1
            else:
                i += 1
        else:
            out.append(ll[i])
            i += 1
    return "\n".join(out)


def findEasiestKey(text, fav, notation=enNotation):
    """
    Find easiest key for song.

        text: song text
        fav: dictionary of (s)favourite chords, with the form {chord: weight}
             (chords in fav are expressed using enNotation)
        return (chord_count, current_key, current_difficulty, easiest_key, easiest_difficulty)
    """
    current_key = autodetectKey(text, notation)
    ws = dict([(k, 0) for k in scales])
    r = re.compile('\[([^]]*)\]')
    count = 0
    for m in r.finditer(text):
        count += 1
        chord = translateChord(m.group(1), notation)
        for k in ws:
            c = transpose(current_key, k, chord)
            if c in fav:
                ws[k] += fav[c]
    easiest_key = current_key
    m = ws[easiest_key]
    for k in ws:
        if m is None or ws[k] > m:
            easiest_key = k
            m = ws[k]
    difficulty = lambda x: max(0, min(1, (count - x) / float(count)))
    if count > 0:
        return (
                count,
                translateChord(current_key, enNotation, notation),
                difficulty(ws[current_key]),
                translateChord(easiest_key, enNotation, notation),
                difficulty(m),
            )
    return (0, current_key, 0, current_key, 0)


def removeChords(text):
    """
    Remove all chords in (ChordPro) text.

        text: song text
        return: text without chords
    """
    return re.sub('\[([^]]*)\]', "", text)


def removeChordPro(text):
    """
    Remove every ChordPro tag, including chords, from text.

        text: song text
        return: text without ChordPro stuff and chords
    """
    return removeSpuriousLines(re.sub('(\[([^]]*)\]|{[^}]*})', "", text))


def replaceTitles(text, replace_string):
    """
    Replace {t:something} and {tititle:something} commands with a string

    :param text: ChordPro song
    :param replace_string: string to be replaced to titles
    :return:
    """
    return re.sub(r'({t:[^}]*}|{title:[^}]*})', replace_string, text)


def pasteChords(src, dest):
    """
    Paste chords from src to dest

    Remove any existing chords in lines in the first k lines of dest,
    if there are k lines in src.
        src: source chordpro text
        dest: destination text
    """
    ss = src.splitlines()
    sd = dest.splitlines()
    out = []
    m = min(len(ss), len(sd))
    r = re.compile('\[([^]]*)\]')
    for i in range(0, m):
        cd = r.sub("", sd[i])
        for x in r.finditer(ss[i]):
            s = x.start()
            if len(cd) < s:
                cd += "".join([" " for y in range(len(cd), s)])
            cd = cd[:s] + x.group(0) + cd[s:]
        out.append(cd)
    for i in range(m, len(sd)):
        out.append(r.sub("", sd[i]))
    return "\n".join(out)



###############################################################
# Name:             PrefTest.py
# Purpose:     Test preference management module
# Author:         Luca Allulli (webmaster@roma21.it)
# Created:     2009-05-30
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:     GNU GPL v2
##############################################################

from xml.dom import minidom

from .Pref import *


def EtaValidator(eta):
    if eta < 0:
        return True, 0
    elif eta < 120:
        return True, eta
    else:
        return False, 0

class Persona(Preferences):
    def __init__(self, parents = [], gui = None):
        Preferences.__init__(self, parents, gui)



Persona.Register("nome", str)
Persona.Register("cognome", str, lambda: "Allulli")
Persona.Register("eta", int, None, EtaValidator)

Persona.Register("figlio", Persona)

a = Persona()
a.nome = 'Luca'
a.figlio = Persona([a, '..'])

x = XmlSerializer()
x.Serialize(a)

#Patch
def newwritexml(self, writer, indent= '', addindent= '', newl= ''):
    if len(self.childNodes)==1 and self.firstChild.nodeType==3:
        writer.write(indent)
        self.oldwritexml(writer) # cancel extra whitespace
        writer.write(newl)
    else:
        self.oldwritexml(writer, indent, addindent, newl)

if not ('oldwritexml' in minidom.Element.__dict__):
    minidom.Element.oldwritexml = minidom.Element.writexml
    minidom.Element.writexml = newwritexml
#End patch

#print x.dom.toprettyxml()

xml = """<?xml version="1.0" ?>
<pref id="1" ptype="Persona">
        <elem name="figlio">
                <pref id="2" ptype="Persona">
                        <parent id="1"/>
                        <elem name="cognome">
                                <str>Allulli</str>
                        </elem>
                </pref>
        </elem>
        <elem name="cognome">
                <str>Allulli</str>
        </elem>
        <elem name="nome">
                <str>Luca</str>
        </elem>
        <elem name="eta">
                <int>30</int>
        </elem>
</pref>
"""

xp = minidom.parseString(xml)

y = XmlDeserializer(xp, [Persona, Preferences])
k = y.Deserialize()



# -*- coding: utf-8 -*- 

###########################################################################
## Python code generated with wxFormBuilder (version Sep  8 2010)
## http://www.wxformbuilder.org/
##
## PLEASE DO "NOT" EDIT THIS FILE!
###########################################################################

import wx

import gettext
_ = gettext.gettext

###########################################################################
## Class CompositePropertyPanel
###########################################################################

class CompositePropertyPanel ( wx.Panel ):
    
    def __init__( self, parent ):
        wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( -1,-1 ), style = wx.TAB_TRAVERSAL )
        
        bSizer21 = wx.BoxSizer( wx.VERTICAL )
        
        bSizer24 = wx.BoxSizer( wx.HORIZONTAL )
        
        self.m_bitmap1 = wx.StaticBitmap( self, wx.ID_ANY, wx.NullBitmap, wx.DefaultPosition, wx.DefaultSize, 0 )
        bSizer24.Add( self.m_bitmap1, 0, wx.ALL, 5 )
        
        self.label = wx.StaticText( self, wx.ID_ANY, _("label"), wx.DefaultPosition, wx.DefaultSize, 0 )
        self.label.Wrap( -1 )
        bSizer24.Add( self.label, 0, wx.ALIGN_CENTER|wx.ALL, 5 )
        
        self.check = wx.CheckBox( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
        self.check.SetValue(True) 
        bSizer24.Add( self.check, 0, wx.ALIGN_CENTER|wx.ALL, 5 )
        
        bSizer21.Add( bSizer24, 0, wx.EXPAND, 5 )
        
        """
        self.value = self.get_widget()
        bSizer21.Add( self.value, 1, wx.ALIGN_CENTER|wx.BOTTOM|wx.EXPAND|wx.LEFT|wx.TOP, 5 )
        """
        
        self.SetSizer( bSizer21 )
        self.Layout()
        bSizer21.Fit( self )
        
        # Connect Events
        self.check.Bind( wx.EVT_CHECKBOX, self.OnCheck )
    
    def __del__( self ):
        pass
    
    
    # Virtual event handlers, overide them in your derived class
    def OnCheck( self, event ):
        event.Skip()
    




###############################################################
# Name:             PreferencesDialog.py
# Purpose:     Allow user to set preferences
# Author:         Luca Allulli (webmaster@roma21.it)
# Created:     2009-10-02
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:     GNU GPL v2
##############################################################

import wx

from . import i18n
from .PreferencesDialog import PreferencesDialog
from .MyDecoSlider import MyDecoSlider
from .Transpose import *
from .Preferences import get_update_frequencies
from .Globals import glb


_ = wx.GetTranslation


class MyPreferencesDialog(PreferencesDialog):
    def __init__(self, parent, preferences, easyChords):
        self.pref = preferences
        self.frame = self
        PreferencesDialog.__init__(self, parent)

        self.easyChords = easyChords

        self.fontCB.Bind(wx.EVT_TEXT_ENTER, self.OnFontSelected, self.fontCB)
        self.fontCB.Bind(wx.EVT_COMBOBOX, self.OnFontSelected, self.fontCB)

        previewSong = _("{t:My Bonnie}\n\nMy [D]Bonnie lies [G]over the [D]ocean\noh [G]bring back my [A]Bonnie to [D]me!\n\n{soc}\n[D]Bring back, [E-]bring back,\n[A]bring back my Bonnie to [D]me!\n{eoc}")
        self.editor.SetText(previewSong)
        self.editor.SetFont(self.pref.editorFace, self.pref.editorSize)
        self.editor.SetReadOnly(True)
        self.autoRemoveBlankLines.SetValue(self.pref.autoAdjustSpuriousLines)
        self.autoTab2Chordpro.SetValue(self.pref.autoAdjustTab2Chordpro)
        self.autoAdjustEasyKey.SetValue(self.pref.autoAdjustEasyKey)
        if self.pref.locale is None:
            lang = i18n.getLang()
        else:
            lang = self.pref.locale
        for l in glb.languages:
            i = self.langCh.Append(glb.languages[l])
            self.langCh.SetClientData(i, l)
            if lang == l:
                self.langCh.SetSelection(i)
        exts = ["crd", "pro", "chopro", "chordpro", "cho"]
        i = 0
        for e in exts:
            self.extension.Append(e)
            if e == self.pref.defaultExtension:
                self.extension.SetSelection(i)
            i += 1
                
        # Default notation
        for n in self.pref.notations:
            i = self.notationCh.Append(n.desc)
            self.notationCh.SetClientData(i, n.id)
        self.notationCh.SetSelection(0)        

        # Update frequency
        sel = 0
        uf = get_update_frequencies()
        for k in uf:
            i = self.frequency.Append(uf[k])
            self.frequency.SetClientData(i, k)
            if k == self.pref.updateFrequency:
                sel = i
        self.frequency.SetSelection(sel)

        # Easy chords
        simplifyGrid = wx.FlexGridSizer(len(easyChords), 2, 0, 0)
        simplifyGrid.AddGrowableCol(1, 1)
        self.simplifyPanel.SetSizer(simplifyGrid)
        self.simplifyPanel.Layout()

        self.decoSliders = {}
        for k in easyChordsOrder:
            simplifyGrid.Add(wx.StaticText(self.simplifyPanel, wx.ID_ANY, getEasyChordsDescription(easyChords[k]), wx.DefaultPosition, wx.DefaultSize, 0), 0, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 5)
            ds = MyDecoSlider(self.simplifyPanel)
            self.decoSliders[k] = ds
            ds.slider.SetValue(self.pref.GetEasyChordsGroup(k))
            simplifyGrid.Add(ds, 1, wx.EXPAND, 5)

        simplifyGrid.FitInside(self.simplifyPanel)

        #self.simplifyPanel.Refresh()

    def OnFontSelected(self, evt):
        f, s = self.GetFont()
        self.editor.SetFont(f, s)
        evt.Skip()

    def GetFont(self):
        face = self.fontCB.GetValue()
        try:
            s = int(self.sizeCB.GetValue())
        except:
            s = 12
        return (face, s)

    def GetLanguage(self):
        return self.langCh.GetClientData(self.langCh.GetSelection())

    def GetNotation(self):
        return self.notationCh.GetClientData(self.notationCh.GetSelection())

    def GetUpdateFrequency(self):
        return self.frequency.GetClientData(self.frequency.GetSelection())

    def OnOk(self, evt):
        self.pref.editorFace, self.pref.editorSize = self.GetFont()
        self.pref.updateFrequency = self.GetUpdateFrequency()
        l = self.GetLanguage()
        lang = i18n.getLang()
        if l is not None and l != lang:
            msg = _("Language settings will be applied when you restart Songpress.")
            d = wx.MessageDialog(self, msg, _("Songpress"), wx.ICON_INFORMATION | wx.OK)
            d.ShowModal()
        self.pref.Save()
        self.pref.locale = l
        self.pref.SetDefaultNotation(self.GetNotation())
        self.pref.autoAdjustSpuriousLines = self.autoRemoveBlankLines.GetValue()
        self.pref.autoAdjustTab2Chordpro = self.autoTab2Chordpro.GetValue()
        self.pref.autoAdjustEasyKey = self.autoAdjustEasyKey.GetValue()
        for k in self.decoSliders:
            self.pref.SetEasyChordsGroup(k, self.decoSliders[k].slider.GetValue())
        self.pref.defaultExtension = self.extension.GetString(self.extension.GetSelection())
        evt.Skip(True)




# coding: utf-8

###############################################################
# Name:             Globals.py
# Purpose:     Hold global settings
# Author:         Luca Allulli (webmaster@roma21.it)
# Created:     2009-09-04
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:     GNU GPL v2
##############################################################

import os.path
import sys
import shutil

import wx


class Globals(object):
    def __init__(self):
        object.__init__(self)
        current_file = os.path.abspath(__file__)
        self.path = os.path.dirname(current_file)
        self.data_path = None

    def InitDataPath(self):
        sp = wx.StandardPaths.Get()
        portable_config = os.path.join(self.path, 'config.ini')
        if os.path.isfile(portable_config):
            # Monkey-patch StandardPaths instance so that portable config.ini is used
            def my_get(*args, **kwargs):
                return self.path
            sp.GetUserDataDir = my_get

        self.data_path = sp.GetUserDataDir()
        old_config = None
        if os.path.isfile(self.data_path):
            old_config = self.data_path + ".orig"
            shutil.move(self.data_path, old_config)
        if not os.path.exists(self.data_path):
            shutil.copytree(os.path.join(self.path, 'templates', 'local_dir'), self.data_path)
        if old_config is not None:
            # Preserve old config file, but don't use it
            shutil.move(old_config, os.path.join(self.data_path, "config.ini.orig"))

    def AddPath(self, filename):
        return os.path.join(self.path, filename)

    def ListLocalGlobalDir(self, rel_path):
        """
        List both the local (data) and global (program) versions of a directory

        :param rel_path: relative path
        :return: list of absolute paths of files
        """
        out = []
        for f in os.listdir(os.path.join(self.path, rel_path)):
            out.append(os.path.join(self.path, rel_path, f))
        for f in os.listdir(os.path.join(self.data_path, rel_path)):
            out.append(os.path.join(self.data_path, rel_path, f))
        return out

    languages = {
        'en': "English",
        'it': "Italiano",
        'fr': u"Français",
        'es': u"Español",
        'de': u"German",
    }

    translators = {
        'en': "Luca Allulli",
        'it': "Luca Allulli",
        'fr': "Raoul Schmitt",
        'es': "Miguel Dell'Uomini",
        'de': "Axel Schneider",
    }

    default_language = 'en'

    PROG_NAME = 'Songpress'
    VERSION = '1.9.0rc3'
    BUG_REPORT_ADDRESS = 'luca@skeed.it'
    YEAR = '2026'

glb = Globals()



###############################################################
# Name:             FontComboBox.py
# Purpose:     wxWidgets control providing a combo box font
#            selector
# Author:         Luca Allulli (webmaster@roma21.it)
# Created:     2009-08-13
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:     GNU GPL v2
##############################################################

import wx

class MyFontEnumerator(wx.FontEnumerator):
    def OnFacename(self, name):
        if name[0] != '@':
            self.names.append(name)
        return True
        
    def Enumerate(self):
        self.names = []
        self.EnumerateFacenames()
        return self
        
    def AppendToComboBox(self, cb):
        self.names.sort(key=lambda x: x.upper())
        cb.AppendItems(self.names)

class FontComboBox(wx.ComboBox):
    def __init__(self, parent, id=-1, defaultFont="Arial"):
        s = wx.Size()
        s.width = 200
        s.height = -1
        wx.ComboBox.__init__(self, parent, id, defaultFont, size=s)
        MyFontEnumerator().Enumerate().AppendToComboBox(self)



# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version Mar 10 2018)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc

# Nothing to localize in this module, because all strings are "dummy" placeholders to be replaced

_ = lambda x: x


###########################################################################
## Class ListDialog
###########################################################################

class ListDialog(wx.Dialog):

    def __init__(self, parent):
        wx.Dialog.__init__(self, parent, id=wx.ID_ANY, title=_(u"Songpress"), pos=wx.DefaultPosition,
                                             size=wx.Size(413, 240), style=wx.DEFAULT_DIALOG_STYLE)

        self.SetSizeHints(wx.DefaultSize, wx.DefaultSize)

        bSizer24 = wx.BoxSizer(wx.VERTICAL)

        self.label = wx.StaticText(self, wx.ID_ANY, _(u"Label text"), wx.DefaultPosition, wx.DefaultSize, 0)
        self.label.Wrap(-1)
        bSizer24.Add(self.label, 0, wx.ALL, 5)

        elementsChoices = []
        self.elements = wx.ListBox(self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, elementsChoices, wx.LB_SINGLE)
        bSizer24.Add(self.elements, 3, wx.ALL | wx.EXPAND, 5)

        m_sdbSizer5 = wx.StdDialogButtonSizer()
        self.m_sdbSizer5OK = wx.Button(self, wx.ID_OK)
        m_sdbSizer5.AddButton(self.m_sdbSizer5OK)
        self.m_sdbSizer5Cancel = wx.Button(self, wx.ID_CANCEL)
        m_sdbSizer5.AddButton(self.m_sdbSizer5Cancel)
        m_sdbSizer5.Realize();

        bSizer24.Add(m_sdbSizer5, 0, wx.EXPAND, 5)

        self.SetSizer(bSizer24)
        self.Layout()

        self.Centre(wx.BOTH)

    def __del__(self):
        pass





# -*- coding: utf-8 -*- 

###########################################################################
## Python code generated with wxFormBuilder (version Sep  8 2010)
## http://www.wxformbuilder.org/
##
## PLEASE DO "NOT" EDIT THIS FILE!
###########################################################################

import wx

import gettext
_ = gettext.gettext

###########################################################################
## Class DecoSlider
###########################################################################

class DecoSlider ( wx.Panel ):
    
    def __init__( self, parent ):
        wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( -1,-1 ), style = wx.TAB_TRAVERSAL )
        
        self.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_WINDOW ) )
        
        bSizer9 = wx.BoxSizer( wx.VERTICAL )
        
        self.slider = wx.Slider( self, wx.ID_ANY, 0, 0, 4, wx.DefaultPosition, wx.DefaultSize, wx.SL_AUTOTICKS|wx.SL_HORIZONTAL )
        bSizer9.Add( self.slider, 0, wx.ALL|wx.EXPAND, 5 )
        
        self.panel = wx.Panel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
        self.panel.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_WINDOW ) )
        self.panel.SetMinSize( wx.Size( -1,30 ) )
        
        bSizer9.Add( self.panel, 1, wx.EXPAND |wx.ALL, 5 )
        
        self.SetSizer( bSizer9 )
        self.Layout()
        bSizer9.Fit( self )
        
        # Connect Events
        self.panel.Bind( wx.EVT_PAINT, self.OnPaint )
        self.panel.Bind( wx.EVT_SIZE, self.OnSize )
    
    def __del__( self ):
        pass
    
    
    # Virtual event handlers, overide them in your derived class
    def OnPaint( self, event ):
        event.Skip()
    
    def OnSize( self, event ):
        event.Skip()
    




###############################################################
# Name:             Renderer.py
# Purpose:     Render a song on a dc
# Author:         Luca Allulli (webmaster@roma21.it)
# Created:     2009-02-21
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:     GNU GPL v2
##############################################################

import wx

from .SongDecorator import *
from .SongTokenizer import *
from .SongFormat import *
from .SongBoxes import *
from .Transpose import translateChord, autodetectNotation
from .EditDistance import minEditDist


class BreakException(Exception):
    pass


class Renderer(object):
    def __init__(self, sf, sd=SongDecorator(), notations=[]):
        object.__init__(self)
        self.text = ""
        self.sd = sd
        self.dc = None
        # SongFormat
        self.starting_sf = sf
        self.sf = sf
        self.format = None
        self.currentBlock = None
        self.currentLine = None
        self.song = None
        self.notation = None
        self.notations = notations
        self.chordPatterns = []

    def BeginBlock(self, type, label=None):
        self.EndBlock()
        if type == SongBlock.verse:
            self.song.verseCount += 1
            if label is None:
                self.song.labelCount += 1
            self.sf.AddVerse()
            self.format = self.sf.verse[-1]
        elif type == SongBlock.chorus:
            self.format = self.sf.chorus
            self.song.chorusCount += 1
        else:
            self.format = self.sf.title
        self.currentBlock = SongBlock(type, self.format)
        self.currentBlock.label = label
        self.currentBlock.verseNumber = self.song.verseCount
        self.currentBlock.verseLabelNumber = self.song.labelCount

    def EndBlock(self):
        if self.currentBlock is not None:
            self.EndLine()
            if self.sf.showChords == 1:
                current = self.currentBlock.chords
                found = False
                for p in self.chordPatterns:
                    led = len(p)
                    med = minEditDist(p, current)
                    if med < led and med < 4:
                        found = True
                        break
                if not found:
                    self.chordPatterns.append(current)
                else:
                    self.currentBlock.RemoveChordBoxes()
            self.song.AddBox(self.currentBlock)
            self.currentBlock = None

    def BeginVerse(self, label=None):
        if self.currentBlock == None:
            self.BeginBlock(SongBlock.verse, label)
            self.label = label

    def BeginChorus(self, label=None):
        self.BeginBlock(SongBlock.chorus, label)

    def ChorusVSkip(self):
        self.EndLine()
        self.BeginLine()
        self.EndLine()

    def AddText(self, text, type=SongText.text):
        if(
            text.strip() != ''
            and type != SongText.title
            and type != SongText.subtitle
            and type != SongText.comment
            and self.currentBlock is not None
            and self.currentBlock.type == SongBlock.title
        ):
            self.EndBlock()
        self.BeginVerse()
        self.BeginLine()
        if type == SongText.comment:
            text = "(" + text + ")"
            format = self.format.comment
        elif type == SongText.chord:
            format = self.format.chord
            if self.sf.showChords == 1:
                self.currentBlock.chords.append(translateChord(text, self.notation, self.notation))
        else:
            format = self.format
        t = SongText(text, format.wxFont, type, format.color)
        if not type == SongText.chord or self.sf.showChords > 0:
            self.currentLine.AddBox(t)

    def AddTitle(self, title):
        if self.currentBlock is None or self.currentBlock.type != SongBlock.title:
            self.BeginBlock(SongBlock.title)
        self.format = self.sf.title
        self.AddText(title, SongText.title)

    def AddSubTitle(self, title):
        if self.currentBlock is None or self.currentBlock.type != SongBlock.title:
            self.BeginBlock(SongBlock.title)
        self.format = self.sf.subtitle
        self.AddText(title, SongText.subtitle)

    def BeginLine(self):
        if self.currentLine == None:
            if self.song.drawWholeSong or self.fromLine <= self.lineCount <= self.toLine:
                self.currentBlock.drawBlock = True
            self.currentLine = SongLine()

    def EndLine(self):
        if self.currentLine is not None:
            self.currentBlock.AddBox(self.currentLine)
            self.currentLine = None

    def GetAttribute(self):
        try:
            tok = self.tkz.next()
            if tok.token != SongTokenizer.colonToken:
                self.tkz.Repeat()
                return None
            tok = self.tkz.next()
            if tok.token != SongTokenizer.attrToken:
                self.tkz.Repeat()
                return ''
            return tok.content
        except StopIteration:
            pass
        return None

    def GetState(self):
        return None if self.currentBlock == None else self.currentBlock.type

    def Render(self, text, dc, fromLine = -1, toLine = -1):
        self.text = text
        self.dc = dc
        self.verseNumber = 0
        self.format = self.starting_sf
        self.currentLine = None
        self.currentBlock = None
        self.song = SongSong(self.sf)
        self.song.drawWholeSong = fromLine == -1
        self.lineCount = -1
        self.fromLine = fromLine
        self.toLine = toLine
        self.sf = SongFormat(self.starting_sf)
        if self.sf.showChords == 1:
            self.notation = autodetectNotation(text, self.notations)
            self.chordPatterns = []

        for l in self.text.splitlines():
            self.lineCount += 1
            state = self.GetState()
            self.tkz = SongTokenizer(l)
            empty = True
            for tok in self.tkz:
                state = self.GetState()
                empty = False
                t = tok.token
                if t == SongTokenizer.normalToken:
                    self.AddText(tok.content)
                elif t == SongTokenizer.chordToken:
                    self.AddText(tok.content[1:], SongText.chord)
                elif t == SongTokenizer.commandToken:
                    cmd = tok.content.lower()
                    if cmd == 'soc' or cmd == 'start_of_chorus':
                        a = self.GetAttribute()
                        self.BeginChorus(a)
                    elif (cmd == 'eoc' or cmd == 'end_of_chorus') and state == SongBlock.chorus:
                        self.EndBlock()
                    elif cmd == 'c' or cmd == 'comment':
                        a = self.GetAttribute()
                        if a is not None:
                            self.AddText(a, SongText.comment)
                    elif cmd == 't' or cmd == 'title':
                        a = self.GetAttribute()
                        if a is not None:
                            self.AddTitle(a)
                    elif cmd == 'st' or cmd == 'subtitle':
                        a = self.GetAttribute()
                        if a is not None:
                            self.AddSubTitle(a)
                    elif cmd == 'verse':
                        self.BeginVerse(self.GetAttribute())
                    elif cmd == 'textsize':
                        try:
                            a = self.GetAttribute()
                            if a is None:
                                size = self.starting_sf.size
                            else:
                                a = a.strip()
                                try:
                                    if a.endswith("%"):
                                        perc = int(a[:-1])
                                        size = self.format.size * perc / 100
                                    else:
                                        size = int(a)
                                except TypeError:
                                    raise BreakException()
                                except ValueError:
                                    raise BreakException()
                            self.format = ParagraphFormat(self.format)
                            self.format.size = size
                            self.sf.size = size
                            self.sf.chorus.size = size
                            self.sf.title.size = size
                            self.sf.subtitle.size = size
                        except BreakException:
                            pass
                    elif cmd == 'textfont':
                        try:
                            face = self.GetAttribute()
                            if face is None:
                                face = self.starting_sf.face
                            else:
                                face = face.strip()
                            self.format = ParagraphFormat(self.format)
                            self.format.face = face
                            self.sf.face = face
                            self.sf.chorus.face = face
                            self.sf.title.face = face
                            self.sf.subtitle.face = face
                        except BreakException:
                            pass
                    elif cmd == 'textcolour':
                        try:
                            color = self.GetAttribute()
                            if color is None:
                                color = self.starting_sf.color
                            else:
                                color = color.strip()
                            self.format = ParagraphFormat(self.format)
                            self.format.color = color
                            self.sf.color = color
                            self.sf.chorus.color = color
                            self.sf.title.color = color
                            self.sf.subtitle.color = color
                        except BreakException:
                            pass
                    elif cmd == 'chordsize':
                        try:
                            a = self.GetAttribute()
                            if a is None:
                                size = self.starting_sf.chord.size
                            else:
                                a = a.strip()
                                try:
                                    if a.endswith("%"):
                                        perc = int(a[:-1])
                                        size = self.format.chord.size * perc / 100
                                    else:
                                        size = int(a)
                                except TypeError:
                                    raise BreakException()
                                except ValueError:
                                    raise BreakException()
                            self.format = ParagraphFormat(self.format)
                            self.format.chord.size = size
                            self.sf.chord.size = size
                            self.sf.chorus.chord.size = size
                        except BreakException:
                            pass
                    elif cmd == 'chordfont':
                        try:
                            face = self.GetAttribute()
                            if face is None:
                                face = self.starting_sf.chord.face
                            else:
                                face = face.strip()
                            self.format = ParagraphFormat(self.format)
                            self.format.chord.face = face
                            self.sf.chord.face = face
                            self.sf.chorus.chord.face = face
                        except BreakException:
                            pass
                    elif cmd == 'chordcolour':
                        try:
                            color = self.GetAttribute()
                            if color is None:
                                color = self.starting_sf.chord.color
                            else:
                                color = color.strip()
                            self.format = ParagraphFormat(self.format)
                            self.format.chord.color = color
                            self.sf.chord.color = color
                            self.sf.chorus.chord.color = color
                        except BreakException:
                            pass

            self.EndLine()
            if empty:
                if state in {SongBlock.verse, SongBlock.title}:
                    self.EndBlock()
                elif state == SongBlock.chorus:
                    self.ChorusVSkip()
        self.EndBlock()
        w, h = self.sd.Draw(self.song, dc)
        self.dc = None
        return w, h

    def SetDecorator(self, sd):
        self.sd = sd



#!/usr/bin/env python

import wx

from .UpdatePanel import UpdatePanel


class MyUpdatePanel(UpdatePanel):
    def __init__(self, parent, preferences, version, description, downloadUrl):
        UpdatePanel.__init__(self, parent)
        self.parent = parent
        self.preferences = preferences
        self.downloadUrl = downloadUrl
        self.v = version
        self.version.SetValue(version)
        self.new_features.SetPage(description)
        
    def __Hide(self):
        self.Show(False)
        self.parent.RemoveChild(self)
        
    def OnDownload(self, evt):
        wx.LaunchDefaultBrowser(self.downloadUrl)
        self.__Hide()
        evt.Skip()
        
    def OnSkip(self, evt):
        self.preferences.ignoredUpdates.add(self.v)
        self.__Hide()
        evt.Skip()
        
    def OnRemind(self, evt):
        self.__Hide()
        evt.Skip()

